/*
 my32bqd.h
  interface for portability with classic Mac graphics names and the GDSpec type

    This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.

    You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __MY32BQD_H__
 #define __MY32BQD_H__

 #if PRAGMA_ONCE
  #pragma once
 #endif

 #ifndef __MY_CSERIES_H
  #include "../my_cseries.h"
 #endif

 #if TARGET_OS_MAC
 /* the old code is obsolete and must be discarded for mgraf for Mac OS */
  #define region16 RgnHandle
 #else
  typedef struct SLRgn	*region16;
  #define NewRgn() newRgn(0)
  #define myLockPixels(p) lockPixels(p->map)
  #define myUnlockPixels(p) unlockPixels(p->map)
  #define QDError() 0
 #endif

enum
{ /* GDSpec "state" values */
	gdSStateBlank		= 0,
	gdSStateDefined		= 1,
	gdSStateInitialized	= -1
};

typedef struct/*(16+)*/
{
	platform_gdevice	gd;
/*
  "state" must be "gdSStateDefined" and "gd" must be a positive value in order
 for the GDHandle to be valid.  Otherwise, it has to be calculated for every
 function that uses "gd".
*/
	int8		state, bit_depth;
	int16		flags, width, height;
	int32		slot;
} GDSpec, *GDSpecPtr;

void myGetGWorld(platform_grafport*,
	platform_gdevice*);
void mySetGWorld(platform_grafport,
	platform_gdevice);
int myNewGWorld(platform_grafport*,
	short pixelDepth,
	const rectangle*,
	const cTab64*);
void myDisposeGWorld(platform_grafport);
int myUpdateGWorld(platform_grafport*,
	short pixelDepth,
	const rectangle*,
	const cTab64*);

void myHideMenuBar(platform_gdevice);
void myShowMenuBar(void);
boolean myMenuBarHidden(void);

/* ----  GDSpec Ops. */

enum
{
	matchSlot		= 0x1,
	matchDepth		= 0x2,
	matchDimensions		= 0x4,

	defineGDSpec		= 0x80	// sets the GDHandle and the state to gdSStateDefined
};

#ifdef HAVE_OS
// isWholeColor represents 8bpp or higher
platform_gdevice IterateGDSpec(GDSpec*,
	platform_gdevice *curGD,	// can be NULL
	uint32 flags,
	boolean *isWholeColor);		// can be NULL

#define FindGDSpecDevice(x,y)		IterateGDSpec(x,NULL,y,NULL)
#define MatchGDSpec(x)			FindGDSpecDevice(x,-1)
#define BestDevice(x)			FindGDSpecDevice(x,matchDepth)

void BuildGDSpec(GDSpec*,
	platform_gdevice);
/* the GDHandle must be set in order for these to work
  you can use IterateGDSpec with the initializeGD flag set */
boolean EqualGDSpec(const GDSpec*,
	const GDSpec*);
void SetDepthGDSpec(GDSpec*);
boolean HasDepthGDSpec(GDSpec*);

boolean display_device_dialog(GDSpec*);
#endif


/* ----- drawing */

typedef enum
{
	_general_overlay,
	_general_text,
	_general_subtraction,
	_adding_overflow,
	_subtracting_overflow,
	_light_on_dark_text,

	NUMBER_OF_ALPHA_CHANNEL_STYLES
} _alpha_channel_style;

void ARGB32ForeColor(ARGB32 _color,
	_alpha_channel_style style);
void RGB32BackColor(ARGB32 _color);

void FixMoveTo(ifixed x,
	ifixed y);
void FixLineTo(ifixed x,
	ifixed y);

#if TARGET_OS_MAC
 #define mSrcCopy srcCopy
 #define mSrcOr srcOr
 #define tSrcOr srcOr
 #define mSubPin subPin
 #define mAddOver addOver
 #define mSubOver subOver
 #define mBlend blend
 #define MSetPort MacSetPort
 #define RawBackColor(c) BackColor(c)
 #define RawForeColor(c) ForeColor(c)

void *get_pixmap_baseaddr(struct PixMap*);
#else
 extern platform_grafport _curPort;
 #include "..\mos\mtext.h"
 typedef struct Point
 {
	int16		v, h;
 } Point;
 #define kFontIDGeneva 3
 #define kFontIDMonaco 4
 #define tSrcOr mTransparent
 #define teForceLeft _special_align_left
 #define teJustCenter _special_align_center
 #define RGBColor rgb48
 #define Rect rect16
 #define Pattern uint8
 #define ColorSpec cSpec
 typedef void *PicHandle, *Handle;
 #define DisposeRgn(r) free(r)
 #define MacInsetRect(r,x,y) OutsetRect16(r,-(x),-(y))
 #define SetClip(r) (void)copyRgn(&_curPort->clipRgn,r)
 #define GetClip(r) (void)copyRgn(&r,_curPort->clipRgn)
 #define OpColor(c) (void)(_curPort->opColor=*(c))
 #define RGBForeColor(c) RGBFore(_curPort,c)
 #define RGBBackColor(c) RGBBack(_curPort,c)
 #define GetBackColor(c) (void)(*(c)=_curPort->bkColor)
 #define GetForeColor(c) (void)(*(c)=_curPort->fgColor)
 #define RawBackColor(c) do{int _bd=_curPort->map->pSize; \
 	_curPort->bkSolid=((c)&pDepthK[_bd])*pSizeMasks[_bd];}while(0)
 #define RawForeColor(c) do{int _bd=_curPort->map->pSize; \
 	_curPort->fgSolid=((c)&pDepthK[_bd])*pSizeMasks[_bd];}while(0)
 #define PenMode(m) (void)(_curPort->pnMode=m)
 #define PenPat(p) rPenPat(_curPort,p)
 #define PenNormal() rPenNormal(_curPort,NULL)
 #define TextMode(m) (void)(_curPort->txMode=m)
 #define TextFace(y) rTextStyle(_curPort,y)
 #define TextSize(y) rTextSize(_curPort,y)
 #define TextFont(i) rTextFont(_curPort,i)
 #define NewPixMap() newPixMap(8,1,1)
 #define DisposePixMap disposePixMap
 #define SetOrigin(i,j) (void)(_curPort->origin.x=i,_curPort->origin.y=j)
 #define myGetGWorldPixMap(p) (0+(p)->map)
 #define myGetPixBaseAddr(p) lockPixels((p)->map)

 /* screen_drawing.c */
void MSetPort(platform_grafport);
#define SetPortWindowPort(w) MSetPort(&(w)->port)
#define SetPortDialogPort(w) MSetPort(&(w)->port)
void GetPort(platform_grafport*);

void ClipRect(const rect16*);

void MoveTo(short x,
	short y);
void MacLineTo(short x,
	short y);
void PenSize(short h,
	short v);
void MacOffsetRect(Rect *rect,
	short dx,
	short dy);
void PaintRect(const Rect *box);
void EraseRect(const Rect *box);
void MacFrameRect(const Rect *box);
void FrameRoundRect(const Rect *box,
	short dx,
	short dy);
short StringWidth(const uint8*);
void DrawString(const uint8*);
short TextWidth(const void *text,
	short pos,
	short len);
short CharWidth(short ch);
void TETextBox(const void *text,
	long length,
	const Rect *box,
	short just);
/* images.c */
void DrawPicture(PicHandle,
	const Rect *frame);
PicHandle GetPicture(short ID);
#endif

const Rect *GetPicFrame(PicHandle h,
	Rect *b);

#endif /*!defined(__MY32BQD_H__)*/
