/*
 myDialogs.h
  interface for portability with classic Mac dialog names

  Revised: 2023.08.12

    This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.

    You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

2015.07.13
*/

#ifndef __MYDIALOGS_H__
 #define __MYDIALOGS_H__

 #define HAVE_OS
 #include "my_cseries.h"

/* ---- constants */

enum
{ /* control states */
	CONTROL_INACTIVE,
	CONTROL_ACTIVE
};

enum
{ /* "ask_save" result codes */
	_user_canceled,
	_user_dont_save,
	_user_save
};

/* ------- prototype thingys */

#if TARGET_OS_MAC
 #include <Dialogs.h>
ModalFilterUPP get_general_filter_upp(void);
pascal Boolean general_filter_proc(DialogRef,
	struct EventRecord*,
	DialogItemIndex*);
#elif !defined(EXTERN_API)
 #include "..\mos\mevent.h"
 #include "..\mos\mwind.h"
 #ifndef __MY32BQD_H__
  #include "my32bqd.h"
 #endif
  typedef signed short	DialogItemIndex;
 enum{menuItemNotFoundErr = -5622};
 enum{overlayDITL};

enum
{
	kUserDialogItem,
	kHelpDialogItem			= 1,
	kControlDialogItem		= 4,
	kButtonDialogItem		= kControlDialogItem,
	kCheckBoxDialogItem		= kControlDialogItem|1,
	kRadioButtonDialogItem		= kControlDialogItem|2,
	kResourceControlDialogItem	= kControlDialogItem|3,
	kStaticTextDialogItem		= 8,
	kStaticTextRightJustItem	= 9, /* non-Mac/QT */
	kEditTextDialogItem		= 16,
	kEditTextScrollableItem		= 17, /* non-Mac/QT */
	kEditTextReadOnlyScrollableItem	= 18, /* non-Mac/QT */
	kIconDialogItem			= 32,
	kPictureDialogItem		= 64,
	kItemDisableBit			= 128
};

 typedef boolean (*ModalFilterUPP)(DialogRef, EventRecord*, DialogItemIndex *itemHit);

boolean general_filter_proc(DialogRef,
	EventRecord*,
	DialogItemIndex*);
 #define get_general_filter_upp() general_filter_proc
#endif

typedef void (*dialog_header_proc)(DialogRef dialog,
	rectangle *frame);
typedef int16 (*dialog_key_proc)(DialogRef dialog,
	struct EventRecord *event);

ControlRef get_dialog_control(DialogRef dialog,
	short item);

#if !TARGET_API_MAC_CARBON
 #if TARGET_OS_MAC
Handle GetControlDataHandle(ControlRef control);
 #else
void *GetControlDataHandle(ControlRef control);

boolean GetColor(Point where,
	const uint8 *prompt,
	const RGBColor *inColor,
	RGBColor *outColor);

long DeleteMenuItems(platform_menu menu,
	uint16 first,
	uint16 count);
void GetDialogItem(DialogRef dlg,
	short item,
	short *outtype,
	void **outctl,
	Rect *outbounds);
void SetDialogItem(DialogRef dlg,
	short item,
	short type,
	void *ctl,
	const Rect *bounds);
void HideDialogItem(DialogRef dlg,
	short item);
void ShowDialogItem(DialogRef dlg,
	short item);
void GetDialogItemText(void *h,
	uint8 *pstr);
void SetDialogItemText(void *h,
	const uint8 *pstr);

int16 GetDialogKeyboardFocusItem(DialogRef dlg);

void SetDialogDefaultItem(DialogRef dlg,
	DialogItemIndex itemHit);

#define SelectDialogItemText(dlg,item,selStart,selEnd) \
	(void)wSelectControlText(wGetIndexedControl(dlg,item-1),selStart,selEnd)
#define FindDialogItem(dlg,pt) \
	wFindControlIndex(dlg,pt,FALSE)

void ModalDialog(ModalFilterUPP filter,
	int16 *itemHit);

void AppendDITL(DialogRef dlg,
	DialogTHndl DITL,
	int16 method);
void ShortenDITL(DialogRef theDialog,
	int16 numberItems);
void DisposeDialog(DialogRef theDialog);

  /* wIdleControls always tracks the cursor */
  #define SetDialogTracksCursor(d,v) (void)(d,v)
  #define NewModalFilterUPP(r) (r)
  #define DisposeModalFilterUPP(r) (void)(r)
  #define ModalDialog wModalDialog
  #define CountMenuItems(m) GetMenuItemCount(m)
  #define CountDITL(w) wCountControls(w)
  #define DrawDialog(w) wDrawControls(w)
 #endif
platform_menu GetControlPopupMenuHandle(ControlRef control);

int dialog_recalc(DialogRef dialog);
#endif

platform_menu get_dialog_popup_menu(DialogRef dialog,
	int16 index);
int setup_strlist_popupmenu(DialogRef d,
	int16 item,
	int16 value,
	int16 ID);

int get_dialog_control_value(DialogRef dialog,
	short item);

DialogRef myGetNewDialog(int16,
	void *dStorage,
	WindowRef behind,
	int32 refCon);

/* this newer one uses INT_NO_VALUE instead of NONE for not setting a parameter */
int setcontrolitem(DialogRef,
	int item,
	int value,
	int minimum,
	int maximum,
	int hilite,
	const uint8 *title);
void modify_control(DialogRef dialog,
	int16 index,
	short hilite,
	short value);
void modify_radio_button_family(DialogRef,
	int16 i1,
	int16 i2,
	int16 which);

int hit_dialog_button(DialogRef d,
	int16 i);

void set_dialog_header_proc(dialog_header_proc);
void set_dialog_key_proc(dialog_key_proc);

dialog_header_proc get_dialog_header_proc(void);
dialog_key_proc get_dialog_key_proc(void);

int ask_save(const uint8 *object_name);
#endif
