/*
 mblt.h
  for the more low-level routines

 * You can redistribute this file and/or modify it under the
 * terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.  You should have
 * received a copy of the license along with this file.

 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See LICENSE.txt more details.
 */

/* ---- constants */

enum
{ /* cnvParam direct-pixel mask & shift table indices */
	_cvdi,
	_cvdj,
	_cvki,
	_cvkj,
	_cvsi,
	_cvsj,
	_cvsg,
	_cvrb,
	_cvmg,
	_cvmr,
	_cvub,
	_cvug,
	_cvur,
	_cvhb,
	_cvhg,
	_cvhr,
	_cvdSize
};

/* per line in _blt_f_rtn: sld, cpy, rem, cnv; four lines per depth */
#define rtnLine			20

/* conversion codes (without smoothing) */
#define indexedMerge		48
#define directMerge32		(48+4)
#define directMerge16		(48+9)

/* ---- typedefs */

#if envx86
 #define DCL_BLT_RTN		__fastcall
#else
 #define DCL_BLT_RTN
#endif
#define DEF_BLT_RTN(n)		void DCL_BLT_RTN n(int32 o, int32 c, bltParam *f)
#define TYP_BLT_RTN(n)		void(DCL_BLT_RTN*n)(int32 o, int32 c, struct bltParam*)

#define DEF_CNV_RTN(n)		uint32 n(cnvParam *f, int o)
#define TYP_CNV_RTN(n)		uint32(*n)(struct cnvParam*, int)

#define DEF_FLS_RTN(n)		void n(cnvParam *f, const int8 *base, ufixed sy)
#define TYP_FLS_RTN(n)		void(*n)(struct cnvParam*, const int8 *base, ufixed sy)

/* alignment is done manually here */
#if (__SC__>0x800 || __MRC__ || macintosh)
 #pragma options align=mac68k
#else
 #pragma pack(push,2)
#endif

typedef struct cnvParam/*(128+)*/
{
	int32		cache[8]; /* pixel and scaling cache */
	int16		hx, hy;
	ifixed		rx; /* scaling rate */
	int32		cm, mp; /* complement mask ('not' mode), pitch */
	TYP_CNV_RTN(rtn);
	void		*t; /* 16 or 32-bit (always 32-bit when 'rtn' here is used) */
	const void	*s, *e; /* source data, end data */
	const uint8	*itb;
	TYP_CNV_RTN(scv);
	TYP_FLS_RTN(ssv);
	const void	*fb;
	const int8	*fx; /* filter table */
	const int8	*fa, *fe; /* filter start and end lines - sometimes ==s */
	uint32		m[5]; /* filter multipliers */
	ifixed		ry; /* scaling rate */
	ifixed		sx; /* internal origin */
	uint16		sw; /* clip width */
	uint8		cc, ac; /* conversion & allocation code */
} cnvParam;

typedef struct bltParam
{
	rect32		cl; /* clip - may be narrower than the port clip */
/*x10*/	int32		ox, oy, oz; /* abs origin/size */
	int32		vx, vy, vz; /* src position/value */
	int32		sc; /* acting solid color pen */
	uint32		opr, opg, opb; /* r/g/b masks */
/*x38*/	cnvParam	cnv;
	int16		bmr, bmg, bmb; /* color magnitudes */
	uint32		mx, mh; /* pattern x-mask & height */
	const void	*sb; /* src base (non-solid) */
	int8		*p, *a, *e; /* dst pixels start, current and absolute end */
	TYP_BLT_RTN(rtn); /* vy is graphical-y when rtn is called */
	struct mlvParam	*mlc; /* midlevel convert-process-convert (e.g., dithering) */
	const uint16	*dtb; /* indexed modes only */
	/* region clipping fields */
	const int16	*ra, *re;
	TYP_BLT_RTN(rrtn); /* region-deferred blitter */
	int32		rr[2]; /* cached current line range/slab */
} bltParam;

#if (__SC__>0x800 || __MRC__ || macintosh)
 #pragma options align=reset
#else
 #pragma pack(pop)
#endif

typedef struct mlvParam
{
	cnvParam	m2d, d2m;
	TYP_BLT_RTN(rtn);
	TYP_BLT_RTN(w2d);
	/* line pixel data follows */
} mlvParam;

typedef struct trapezoid
{
	int64		x1, dx1; /* manual point-x/slope */
	bltParam	prm;
} trapezoid;

typedef struct RRectParam
{
	const int8	*ua, *ue; /* full-width markers */
	int64		dy, y0, y1; /* y-inc, top of rect-rounding positions */
	ufixed		rw, mw; /* round & mid-width; rarcmul(y,rw)+mw */
} RRectParam;

typedef TYP_BLT_RTN(bltRoutine);
typedef TYP_CNV_RTN(cnvRoutine);

/* ---- globals */

extern const int8 *mg_filterQ6lut[];

/* ---- macros, inlines */

/* 3 bits per channel minimum; only need to double twice (h=3; h'=6; h''=12) */
#define cnvDirect(v,u,t,r)	\
	(u=v<<t[_cvub], u|=u>>t[_cvsi], u|=u>>t[_cvdi], r=u>>t[_cvhb], \
	u=v<<t[_cvug]&t[_cvkj], u|=u>>t[_cvsj], u|=u>>t[_cvdj], r|=u>>t[_cvhg]&t[_cvmg], \
	u=v<<t[_cvur]&t[_cvki], u|=u>>t[_cvsi], u|=u>>t[_cvdi], r|=u>>t[_cvhr]&t[_cvmr])
#define cnvDirect32(v,u,t,r)	\
	(u=v<<t[_cvub], u|=u>>t[_cvsi], r=u>>t[_cvhb], \
	u=v<<t[_cvug]&t[_cvkj], u|=u>>t[_cvsj], r|=u>>t[_cvhg]&t[_cvmg], \
	u=v<<t[_cvur]&t[_cvki], u|=u>>t[_cvsi], r|=u>>t[_cvhr]&t[_cvmr])

#define cnvGray(v,u,t,r)	\
	(u=(v<<t[_cvur]&t[_cvki])>>16, u|=u>>t[_cvsi], u|=u>>t[_cvdi], \
	r=(v<<t[_cvug]&t[_cvkj])>>16, r|=r>>t[_cvsj], r|=r>>t[_cvdj], \
	v=v<<t[_cvub]>>16, v|=v>>t[_cvsi], v|=v>>t[_cvdi], formGray(u,r,v,u))
#define cnvGray32(v,u,t,r)	\
	(u=(v<<t[_cvur]&t[_cvki])>>16, u|=u>>t[_cvsi], \
	r=(v<<t[_cvug]&t[_cvkj])>>16, r|=r>>t[_cvsj], \
	v=v<<t[_cvub]>>16, v|=v>>t[_cvsi], v|=v>>t[_cvdi], formGray(u,r,v,u))

#define adjustHLine1(f,o,c)	do{ \
	if(c>f.cl.right) c=f.cl.right; \
	if(o<f.cl.left) o=f.cl.left; \
	c-=o; \
	if((o-=f.ox)<0) c+=o, o=0; \
}while(0)
/* two counts, adjusted with offset */
#define adjustHLine2(f,o,c,c2)	do{ \
	if(c>f.cl.right) c=f.cl.right; \
	if(c2>f.cl.right) c2=f.cl.right; \
	if(o<f.cl.left) o=f.cl.left; \
	c-=o,c2-=o; \
	if((o-=f.ox)<0) c+=o,c2+=o,o=0; \
}while(0)

/* initialization */
__forceinline int rInitClip(const bltParam *f, /* must be valid, locked */
	int32 o,
	int8 **a)
{
	int32		c;

	*a = f->p, o-= c=f->cl.top;
	if (o>0)
		c+= o, o = 0;
	*a+= f->oz*(c-f->oy);
	return -o;
}

__forceinline int rInitClip2(const bltParam *f, /* must be valid, locked */
	int32 o,
	int32 c,
	int8 **a,
	int8 **e)
{
	*e = (*a=f->p)+f->oz*(c-f->oy);
	o-= c=f->cl.top;
	if (o>0)
		c+= o, o = 0;
	*a+= f->oz*(c-f->oy);
	return -o;
}

/* ---- prototypes */

/*
 rTrapezoid
	Renders a trapezoid.
	The prm field must be initialized with rInitBlt.
*/
void rTrapezoid(trapezoid*, /* must be valid */
	pt2dq p0,
	pt2dq p1);
/*
 rInitBlt
	Initializes a bltParam, with vertical clipping.
	It calls rGetBltRtn, and lockPixels as necessary.
  IMPORTANT:
	rEndBlt must be used when done.
	When merge modes are used in scaling, sw and ry must be already defined.
  Result:
	The clipping y-skip, or -1 on error or hollow.
*/
/* returns clipping y-skip */
int rInitBlt(bltParam *f, /* must be valid */
	grafPort *p, /* must be valid */
	const void *s, /* optional source data (otherwise the pen pattern/color) */
	const cTab64 *t, /* optional source table */
	int y0,
	int y1,
	int mode); /* OR in `formPxFmt(m)<<8` when passing src img data */
void rEndBlt(bltParam *f);
/*
 rEndLine
	In the case of source data, call this when a line ends.
  Result:
	Zero if f->s hit 'e'.
*/
int rEndLine(bltParam *f,
	const int8 *e);
/*
 rBltOrigin
	Initializes the image/pattern origin for a scanline.
*/
void rBltOrigin(bltParam *f,
	int x,
	int y);
/*
 rInitCnv
	Initializes a conversion routine and possible table (in f->t).
	fmt has additional source format bits:
	 #15=has horiz. scaling; #14=request upscale filter.
*/
int rInitCnv(cnvParam *f, /* must be valid; f->ac is assumed zero */
	pixMap *m, /* must be valid */
	const cTab64 *ct,
	int fmt,
	ufixed scale,
	uint8 mode,
	uint8 flag);
void rEndCnv(cnvParam *f);

mlvParam *rInitDithering(pixMap *dm);

/*
 rBlt
	Transfers an image set up with rInitBlt.
	The validity of f->s should be consistent with 's' passed to rInitBlt.
	f->cnv.ry must be set to the vertical scaling rate.
	If f->cnv.ry is all ones (~0), upward (set s/d to first line past end).
	For horizontal scaling, cnv.sw needs to be set to source width.
*/
int rBlt(bltParam *f,
	int pitch,
	int width,
	int x,
	int sx,
	int sy,
	int ky);

/*
 rInitRoundRect
	Calculates parameters given the measurements given, including first pixel.
*/
void rInitRoundRect(RRectParam *r,
	const bltParam *f,
	int32 ky,
	ufixed w,
	ifixed bt,
	ifixed bb,
	ufixed rx,
	ufixed ry,
	ufixed oy);

/* vertical interpolation routines */
extern DEF_FLS_RTN(hssnone);
extern DEF_FLS_RTN(hssvfir3);
extern DEF_FLS_RTN(hssvbc);
