/*
 mclipboard.c

  Created: 2016.08.02

 * You can redistribute this file and/or modify it under the
 * terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.  You should have
 * received a copy of the license along with this file.

 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See LICENSE.txt more details.
 */

#ifdef WIN32
 #include <windows.h>
 #include <wchar.h>
#endif

#include "mtext.h"

/* ---- code */

#ifdef WIN32
char *GetClipboard_MacRoman(int32 *outlen,
	boolean stripCRLF)
{
	HANDLE		h;
	wchar_t		*s, *p;
	char		*d = NULL;
	int32		z, r, l;

	if (!OpenClipboard(world_screen_window) || (h=GetClipboardData(CF_UNICODETEXT))==0)
		return NULL;
	z = GlobalSize(h);
	if ((s=GlobalLock(h))==NULL)
		{d = NULL; goto end;}
	/* some programs leave garbage after the nul, so a strlen must be computed */
	p = wmemchr(s,0,z);
	if (p)
		z = p-s;
	if (outlen)
		*outlen = z;
	if ((d=calloc(z+1,1))==NULL)
		goto end;
	z = WideCharToMultiByte(CP_MACCP,0,s,z,d,z,0,0);
	if (z>0 && stripCRLF)
	{
		for(l=r=0; r<z-1; ++r,++l)
			if ((d[l]=d[r])==13 && d[r+1]==10)
				++r;
		d[l] = d[r], l+= r!=z;
		if (l<z)
		{
			z = l;
			s = realloc(d,z);
			if (s)
				d = (char*)s;
		}
	}
	if (outlen)
		*outlen = z;
end:	GlobalUnlock(h);
	CloseClipboard();
	return d;
}

int SetClipboard_MacRoman(const char *s,
	int32 z)
{
	HANDLE		h;
	wchar_t		*p;
	int		r, wlen = z*2;

	if (!OpenClipboard(world_screen_window))
		return -4;
	EmptyClipboard();
	if (!s || z<=0)
		return 0;
	if ((h=GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE|GMEM_ZEROINIT,wlen+2))==NULL)
		return -2;
	if ((p=GlobalLock(h))==NULL)
		{r =-1; goto crap;}
	memset(p,0,wlen+2);
	MultiByteToWideChar(CP_MACCP,0,s,-1,p,z);
	GlobalUnlock(h);
	r = 1;
	if(!SetClipboardData(CF_UNICODETEXT,h))
	{
		r = 0;
	crap:	GlobalFree(h);
	}
	CloseClipboard();
	return r;
}

int ClipMenuSelect(uint16 flags,
	short x,
	short y,
	const char *extension)
{
	POINT		lpt;
	HMENU		menu;
	int		r, modStr = flags&tBoxModifiable ? MF_STRING:MF_STRING|MF_GRAYED;
	int		empStr = flags&tNotEmptyState ? MF_STRING:MF_STRING|MF_GRAYED;

	if ((menu=CreatePopupMenu())==NULL)
		return -2;
	AppendMenu(menu,modStr|(flags&tHasSelectionState
		? MF_STRING:MF_STRING|MF_GRAYED),1,"Cut");
	AppendMenu(menu,flags&(tHasSelectionState|tHasWordState)
		? MF_STRING:MF_STRING|MF_GRAYED,2,flags&tHasWordState ? "Copy Word":"Copy");
	AppendMenu(menu,modStr,3,"Paste");
	AppendMenu(menu,modStr|empStr,4,"Delete");
	AppendMenu(menu,MF_SEPARATOR,5,NULL);
	if (extension)
		AppendHMenuItems(menu,extension,flags&tDisableMenuExt ? -1:100);
	AppendMenu(menu,empStr,6,"Select All");

	if (x==tNoXY)
		get_mouse_location(&x,&y,0);
	lpt.x = x, lpt.y = y;
	ClientToScreen(world_screen_window,&lpt);
	r = 0;
	if (TrackPopupMenu(menu,TPM_LEFTALIGN,lpt.x,lpt.y,0,world_screen_window,NULL))
	{
		MSG		msg;

		if (PeekMessage(&msg,world_screen_window,WM_COMMAND,WM_COMMAND,PM_REMOVE))
		{
			TranslateMessage(&msg);
			if (!HIWORD(msg.wParam))
			#if envx64
				r = LOWORD(msg.wParam|msg.wParam>>32);
			#else
				r = LOWORD(msg.wParam);
			#endif
			DispatchMessage(&msg);
		}
	}
	DestroyMenu(menu);
	return r;
}
#endif
