/*
 mimg_file.h

 * You can redistribute this file and/or modify it under the
 * terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.  You should have
 * received a copy of the license along with this file.

 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See LICENSE.txt more details.
 */

//#include <stdio.h>

#ifndef __MGRAF_H
 #include "mgraf.h"
#endif

/* ---- constants */

enum
{ /* disposal constants */
	_gifdisp_solid,
	_gifdisp_keep,
	_gifdisp_erase,
	_gifdisp_restoreback,
	_gifdisp_restoreprev
};

/* ---- typedefs */

#include <pshpack2.h>

/* last frame info */
typedef struct GIF89info
{
	int32		loopCount; /* 0 = infinite; <0 defaults to infinite */
	int32		delayTime; /* in hundredths of a second; <0 = no delay */
	uint8		index;
	int8		inputFlag;
	uint8		transparent;
	uint8		disposal;
} GIF89info;

/* file global state */
typedef struct GIFScreen
{
	uint32		ColorMap[256];
	uint16		Width;
	uint16		Height;
	int		FrameCount;
	int16		Transparent; /* current color; negative on 'none' */
	uint8		Background;
	uint8		ColorResolution;
	uint8		AspectRatio;
	uint8		BPP;
} GIFScreen;

typedef struct GIFState
{
	GIFScreen	screen;
	GIF89info	info;
 #ifdef __LZW_H
	LZWstate	lzw;
 #else
	int16		LZWdata[16550];
 #endif
} GIFState;

typedef struct BMdcmpHeader
{
	cTab64		*ct;
	int		headerSize, headerPad;
	int		w, h;
	ifixed		hRes, vRes;
	uint16		biCompression, biPlanes;
	int16		depth;
	int8		bpp, rowOrder;
	RGBLineInfo	hk;
} BMdcmpHeader;

#include <poppack.h>

/* ---- prototypes */

int imgReadFileDIB(pixMap **m,
	sio_func_t rf,
	const void *s,
	boolean hasBMPHeader);
#define imgReadFileBMP(m,rf,s) imgReadFileDIB(m,rf,s,1)

int dcmpReadFileDIB(pixMap **m,
	sio_func_t rf,
	const void *s,
	BMdcmpHeader *dhd);

#ifndef NO_DCMP_PPM_IMAGE_FILES
int32 PPM_readint(sio_func_t rf,
	const void *s,
	uint8 *inComment);
int imgReadFilePPM(pixMap **m,
	sio_func_t rf,
	const void *s);
#endif

/* mimg_gif.c */

/*
 decompressGIF
	Decompresses a frame into an image buffer.
	If successful, the state will hold the next frame for animation.
	Composite images will be 32-bit with alpha defined.
  NOTE:
	When calling for an accumulated state (for info or multiple frames),
	be sure to set the initial state pointer to NULL, and free() when finished.
*/
int decompressGIF(pixMap **output, /* can be NULL (verify / get frame count) */
	sio_func_t rf,
	const void *s,
	int desiredFrame, /* pass -n to continue from previous frame(s) */
	GIFState **state); /* can be NULL */

/* mimg_jpeg.c */

int decompressJPEG(pixMap **output,
	sio_func_t rf,
	const void *s);
