/*
 mtext.h
  include for all built-in text-related code

   Created: 2014.03.30
    Author: Adam J. P.

 * You can redistribute this file and/or modify it under the
 * terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.  You should have
 * received a copy of the license along with this file.

 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See LICENSE.txt more details.
 */

#ifndef __MTEXT__
 #define __MTEXT__
 #include "mgraf.h"

/* ---- constants */

#ifndef mOS_MaxSequentialFonts
 #define mOS_MaxSequentialFonts	0
#endif
#define mOS_MaxFonts		(mOS_MaxSequentialFonts ? mOS_MaxSequentialFonts:32768)

#ifndef mOS_MaxSequentialFonts
 #define MFONT_NO_UNPACK1BPP 1
#endif

typedef enum
{
	tNoBreak,
	tBreakLine,
	tBreakWord,
	tBreakChar,
	tBreakOverflow
} textBreakCode;

enum
{ /* tBox flags */
	tBoxActive		= 0x1,	/* user can select */
	tBoxModifiable		= 0x2,	/* otherwise read-only, no caret */
	tBoxAutoScroll		= 0x4,	/* view adjusts for full caret visiblility */
	tBoxFrame		= 0x8,	/* draw edit-text frame */
	tBoxAutoVScroll		= 0x10, /* vert. scroll bar installed on right - tBoxNew */
	tBoxAutoHScroll		= 0x20, /* horiz. scroll bar installed on bottom - tBoxNew */
	tBoxDragDrop		= 0x40, /* can use mouse to drag & drop text */
	tHiliteLayer		= 0x80,	/* hilite before text; otherwise, after, inverting */
	tBoxListSelect		= 0x100, /* when read-only, select whole lines */
	tBoxInsertOnly		= 0x200, /* selection clipped to caret/click point */
	tBoxUseCustBgColor	= 0x400,
	tBoxInhibitStyles	= 0x800,
	tBoxCharSpaceFullJust	= 0x1000, /* apply justified spacing to all characters */
	tBoxSetOnDrag		= 0x2000,
	tBoxHardScroll		= 0x4000, /* scroll v/h by full w/h minus 16; half otherwise */
	tBoxInitHideScrolls	= 0x8000,
	/* additional options */
	tBoxInhibitDragDelete	= 0x10000,
	tBoxHideCharacters	= 0x20000, /* password mode; draws only bullet characters */
	/* internal flags */
	tBoxFullUpdate		= 0x800000, /* position has changed; draw more */
	tBoxUDirection		= 0x1000000, /* to avoid oscillation when selecting by mouse */
	tBoxBDirection		= 0x2000000, /* to avoid oscillation when selecting by mouse */
	tBoxWasModified		= 0x4000000,
	tBoxOverwriteState	= 0x8000000,
	tBoxSelectingState	= 0x10000000, /* mouse down, selecting more/less characters */
	tBoxDragState		= 0x20000000, /* amid dragging and dropping text */
	tBoxCaretState		= 0x40000000, /* some visual control when selecting */
	tBoxFocusState		= 0x80000000u,
	/* used by ClipMenuSelect */
	tNotEmptyState		= 0x10,
	tHasWordState		= 0x20,	/* Copy becomes Copy Word */
	tHasSelectionState	= 0x40,
	tDisableMenuExt		= 0x80
};

enum
{
	tNoSelLX		= (uint16)~0u,
	tNoXY			=-1,
	tMenuOnly		= 0xFF00
};

enum
{
	tSearchCaseSensitive	= 0x1,
	tSearchWholeWords	= 0x2,
	tSearchBackwards	= 0x80
};

#ifdef MTEXT_LEGACY_NAMES
/* legacy font IDs that were somewhat reserved */
 #define newYork		2
 #define geneva			3
 #define monaco			4
 #define losAngeles		12
 #define timesFont		20
 #define helvetica		21
 #define courier		22
 #define symbol			23
 /* system font glyphs */
 #define commandMark		0x11
 #define checkMark		0x12
 #define diamondMark		0x13
 #define appleMark		0x14
#endif

#define ffHasfctb		0x80	/* has font color table */
#define ffDepthMask		0x3	/* bpp = 1<<x */

/* ---- typedefs */


#include <pshpack2.h>

typedef struct ffGlyph
{ /* NOTE: bitmap reflects minimized range in [minx,maxx][miny,maxy] */
	uint8		*bitmap;
	int32		index; /* internal index */
	int32		cchar; /* up to Unicode 20-bit */
	ifixed		advance;
	int16		ptSize, h, w;
	int16		top, left;
	int16		miny, maxy;
	int16		minx, maxx;
} ffGlyph;

#ifdef MFONT_TTF
typedef struct TTF_Font
{
	ffGlyph		cache[256];
	ffGlyph		scratch;
#ifdef TTF_OUTLINING
	int		outline;
#endif
#ifdef FT_GLYPH_H
	FT_Face		face;
	FT_Open_Args	args;
#else
	uint8		_faceData[8];
#endif
} TTF_Font;
#endif

typedef struct ffFont
{
	uint8		flag, style;
	int16		ptSize;
	int16		ctCount; /* color case */
	int16		maxWidth, maxKerning;
	int16		ascent, descent, leading;
	uint16		height, bmWidth;
	int32		bmSize, pitch;
	uint32		firstChar, lastChar;
	void		*data;
	int16		*widTable; /* NULL = TTF */
	/* bitmapped data: location table (uint16*) follows (data+bmSize),
	  width table (int16*) directly follows
	  - 'undefined' char @ (lastChar-firstCharacter+1)*2
	*/
#ifdef MFONT_TTF
	TTF_Font	*sfnt;
#endif
} ffFont;

typedef struct ffFamily
{
	int		ID, asscCount;
	ffFont		font[1];
} ffFamily;

typedef struct tStyleRun/*(16)*/
{
	int32		offs;
	int16		font, size;
	uint8		style, special;
	rgb48		color;
} tStyleRun;

typedef struct tBgColorRun/*(8)*/
{
	int32		offs;
	uint32		color; /* 10:11:10 RGB format */
} tBgColorRun;

typedef struct tOldStyleRun/*(20)*/
{ /* scrap handle variant */
	int32		offs;
	int16		height, ascent; /* not used */
	int16		font;
	uint8		style, pad;
	int16		size;
	rgb48		color;
} tOldStyleRun;

typedef struct tLineRun/*(20)*/
{
	int32		offs;
	int		length; /* bit #31 on if end of line */
	int		width, vpos;
	uint16		height;
	int16		descent;
} tLineRun;

#define DEF_TBWBREAK_RTN(n)	int32 n(textBoxObj *tbox, int32 i, boolean reverse)
#define TYP_TBWBREAK_RTN(n)	int32(*n)(struct textBoxObj*, int32, boolean)
#define DEF_TBDRAWLN_RTN(n)	int n(textBoxObj *tbox, int32 i, const tLineRun *lr, int v, int y)
#define TYP_TBDRAWLN_RTN(n)	int(*n)(struct textBoxObj*, int32, const tLineRun*, int, int)
#define DEF_TBRECLLN_RTN(n)	int n(textBoxObj *tbox, tLineRun *lr, int lc, int brkcode, int y)
#define TYP_TBRECLLN_RTN(n)	int(*n)(struct textBoxObj*, tLineRun*, int, int, int)

typedef struct textBoxObj
{
	rect16		bounds, txbounds;
	int32		length;
	int32		selStart, selEnd; /* may be in reverse order */
	int32		selCLMinPos;
	int		scrollV, scrollH;
	int		tHeight; /* total text height */
	int		lineCount, styleCount;
	ufixed		maxWidth; /* working range really */
	ifixed		tabAlign;
	uint16		selLCursX;
	rgb48		hiliteColor, custBgColor;
/*	int		tCellXCount, tCellXSpace, tCellDSize; -- future fields */
	uint32		flags;
	int16		caretTime, dragTime;

	TYP_TBWBREAK_RTN(wordBreak);
	TYP_TBDRAWLN_RTN(drawLine); /* draw also calls this at end with (tbox,result,NULL) */
	TYP_TBRECLLN_RTN(recalLine);
	void		*cbUserData;

	void		*textH;
	char		*textA;
	tLineRun	*lines;
	tStyleRun	*styles;

	struct windowRecord	*wind;
	struct ctlObj		*ctlScrollV, *ctlScrollH;
} textBoxObj;

#include <poppack.h>

/* ---- globals */

extern ffFamily	*fmFonts[mOS_MaxFonts];
extern int16	ffDefault, ffFWDefault;
extern uint8	ffSystem; /* 0 by default */

extern ufixed	_ffPointScale, _ffUlThick, _ffOlThick;

/* ---- prototypes */

extern DEF_TBWBREAK_RTN(_defaultTBWordBreak);
extern DEF_TBDRAWLN_RTN(_custTBStripeBG);

/*
 ffLoad
	Loads an entire font family.
	Returns NULL on error.
*/
ffFamily *ffLoad(struct ResFile_Memory *rMap,
	const uint8 *name,
	int ID);
/*
 ffMatch
	Finds or generates the closest bitmapped font.
*/
ffFont *ffMatch(ffFamily *ff,
	int size,
	int8 styl);
/*
 ffFree
	Frees all memory allocated for a font family.
	All bmfs obtained from this family will be invalid.
*/
int ffFree(ffFamily *ff);

/*
 fmLoadMaster
	Loads the global table from a resource file.
*/
int fmLoadMaster(const char *name,
	int (__cdecl iterator)(const uint8 *name,
		int ID));
int fmLoadAll(struct ResFile_Memory *rmap,
	int (__cdecl iterator)(const uint8 *name,
		int ID));
void fmUnloadAll(void);
int fmFlushCache(ffFamily *ff);

#ifdef MFONT_TTF
int TTF_OpenSFNT(ffFont *fm,
	int face_index);
int TTF_SetPtSize(ffFont *fm,
	int ptsize);
/* caller must dispose of TTdata */
void TTF_CloseFont(ffFont *fm);

ffGlyph *TTF_GetGlyph(ffFont *fm,
	uint32 ch,
	int *rerr,
	boolean get_bmap);
int TTF_GetKerning(ffFont *fm,
	int prevch,
	int nextch,
	ifixed *kerningX,
	ifixed *kerningY);

int TTF_FlushGlyph(ffGlyph *glyph); /* must be valid */
int TTF_FlushCache(TTF_Font *ttf); /* must be valid */
#endif

/* settings */

#ifdef _CRTNOP
_CRTNOP
#endif
int rTextFace(grafPort *p,
	int16 font,
	int16 size,
	uint16 style);
#ifdef _CRTNOP
_CRTNOP
#endif
int rTextFont(grafPort *p,
	int16 font);
#ifdef _CRTNOP
_CRTNOP
#endif
int rTextSize(grafPort *p,
	int16 size);
#ifdef _CRTNOP
_CRTNOP
#endif
int rTextStyle(grafPort *p,
	uint16 style);
#ifdef _CRTNOP
_CRTNOP
#endif
int rSetFScaleDisable(grafPort *p,
	int fscale);

/* measurement, rendering */

/*
 rTextWidth
	Calculates the basic, integral width of a string.
	No position is given; tabs are not used in this calculation.
	If the passed length is negative, strlen is used for length.
*/
int rTextWidth(grafPort *p,
	const char *s,
	int length,
	ufixed x);
/*
 rClipText
	Finds the maximum width-clipped length for a string.
	If the passed length is negative, strlen is used for length.
*/
int rClipText(grafPort *p,
	const char *text,
	int length,
	ufixed *maxWidth,
	ufixed *minWidth, /* result only (<x); can be NULL */
	ufixed x);
/*
 rTextBreak
	Finds the first non-space or line-ending break.
	The resulting maxWidth does not include breaking whitespace.
	Tab origin (if applied) is 0; adjust text position if needed.
  Result:
	The next non-space character or next line; negative on error.
*/
textBreakCode rTextBreak(grafPort *p, /* must be valid */
	const char *lineStart, /* can be NULL */
	const char *pos, /* must be valid */
	int *count, /* must be valid */
	ufixed *maxWidth, /* must be valid */
	ufixed w);

int32 rTextSkipEmpty(const char *s,
	int32 len,
	int32 i,
	boolean reverse);
int32 rTextScanEOL(const char *text,
	int32 len,
	int32 i,
	boolean reverse);
boolean rTextStyleMatch(const tStyleRun *sra,
	const tStyleRun *srb,
	boolean hasLineEndings);

/*
 rGetFontInfo
	Calculates various length measurements for a font, adjusted for scale.
	Bold style adjustment (when no bold font is found) is not added to maxWidth.
*/
#ifdef _CRTNOP
_CRTNOP
#endif
int rGetFontInfo(ffFace *face,
	int16 *maxWidth,
	int16 *ascent,
	int16 *descent,
	int16 *leading,
	int16 *maxKerning);

/*
 rDrawText
	Renders text to a port, given the port's text settings.
	Offscreen imaging is used.
	If the passed length is negative, strlen is used for length.

  Result:
	0 if nothing drawn; next position if positve; IMIN(ifixed) on error.
*/
#ifdef _CRTNOP
_CRTNOP
#endif
ifixed rDrawText(grafPort *p,
	const char*,
	int length,
	int y,
	ifixed x);

/* ---- text boxes */

/*
 tBoxNew
	Creates a new textbox object, copying the string ptr and length.
*/
textBoxObj *tBoxNew(struct windowRecord*,
	const rect16 *bounds,
	const char*,
	int32 length,
	uint16 flags);
#ifdef _CRTNOP
_CRTNOP
#endif
int tBoxDispose(textBoxObj*);

/*
 tBoxGetText
	Makes sure that the text ptr is locked, and returns the text pointer.

  Result:
	The current pointer, or NULL on empty or error.
*/
char *tBoxGetText(textBoxObj *tbox);

/*
 tBoxGetChar
	Makes sure that the text ptr is locked, and returns the character at i.

  Result:
	The character, or -1 on empty or error.
*/
int tBoxGetChar(textBoxObj *tbox,
	int32 i);
int tBoxSetChar(textBoxObj *tbox,
	int32 i,
	int c,
	int update);

/*
 tBoxCopyText
	Creates a text handle and returns the text for the range given.
	Passing a negative for start will be seen as a binary complement, and
	 null-terminate the output for use as a C string.
 	Use tBoxCopyStyles or tBoxCaptureStyles to get a copy of the styles.

  Result:
	The new handle, or NULL on empty or error (check length>0).
*/
void *tBoxCopyText(textBoxObj *tbox,
	int32 start,
	int32 end);

/*
 tBoxSetTextRaw
	Sets the text data to given parameters, with styleRuns untouched.

  Result:
	1 on success; zero if now empty.
*/
int tBoxSetTextRaw(textBoxObj *tbox,
	const char *s,
	int32 z);

/*
 tBoxRecalc
	Calculates the line-runs in a textbox.
	An optional starting line can speed up insertion.
  Result:
	1 on success; zero if empty.
*/
int tBoxRecalc(textBoxObj*,
	int startLine); /* zero-based, pinned to existing range */
/*
 tBoxDraw
	Renders the textbox, erasing the back first (using bkColor).
	It will use tBoxRecalc if necessary.
  Result:
	1 on success; zero if empty.
*/
#ifdef _CRTNOP
_CRTNOP
#endif
int tBoxDraw(textBoxObj*);

/*
 tBoxUpdate
	Draws and updates the visible portion of the native window.
	Optionally, the scroll ranges/positions can be updated.
*/
#ifdef _CRTNOP
_CRTNOP
#endif
int tBoxUpdate(textBoxObj *tbox,
	const rect16 *area, /* can be NULL */
	int firstLine); /* pass negative for no recalc; IMAX for scrolls only */

/*
 tBoxScroll
	Adjusts the textbox scrolls, and updates if changed.
	The new values are pinned to the practical range (tBoxMaxScroll).
*/
int tBoxScroll(textBoxObj*,
	int amtv,
	int amth);
int tBoxMaxScroll(const textBoxObj*,
	int32 *maxv,
	int32 *maxh);
int tBoxUpdateScrollLimits(textBoxObj *tbox);

/*
 tBoxSelect
	Sets the textbox selection, and updates if changed.
	The new values are pinned to the functional range.
*/
#ifdef _CRTNOP
_CRTNOP
#endif
int tBoxSelect(textBoxObj*,
	int start,
	int end);
/*
 tBoxClick
	Handles cursor selection, dragging.
*/
int tBoxClick(textBoxObj*,
	pt2d16,
	uint16 eventModifiers,
	boolean dblClick);
/*
 tBoxIdle
	If active and writable, the caret is updated.

	NOTE: this routine is set up to unlock the text handle.
*/
#ifdef _CRTNOP
_CRTNOP
#endif
int tBoxIdle(textBoxObj*);
/*
 tBoxKey
	If active and writable, handles a key function (may insert a char).
	Pass the event message cast as a uint32 for key.
*/
#ifdef _CRTNOP
_CRTNOP
#endif
int tBoxKey(textBoxObj*,
	uint32 key,
	uint16 eventModifiers);
/*
 tBoxDelete
	If active and writable, removes text and styles within a range.
	The range can run in reverse (start after end).
*/
#ifdef _CRTNOP
_CRTNOP
#endif
int tBoxDelete(textBoxObj*,
	int32 start,
	int32 end,
	int update);
/*
 tBoxInsert
	Inserts text and/or optional styling information.
	A styleRun and no text can be inserted at the end.
	Passing a negative value for insLength will call strlen.
*/
#ifdef _CRTNOP
_CRTNOP
#endif
int tBoxInsert(textBoxObj*,
	int32 start,
	const char *insText,
	int32 insLength,
	const tStyleRun *insStyles,
	int insStyleCt,
	int update); /* pass negative to avoid drawing altogether */

/*
 tBoxImport
	Imports text and styling from Classic Mac OS 'styl' data.
*/
int tBoxImport(textBoxObj *tbox,
	int32 start,
	const char *s,
	int32 length,
	const tOldStyleRun *styles, /* big endian */
	int runCount,
	int update);
/*
 tBoxPinView
	Adjusts the scrolls so selEnd is in view.
  Note:
	This does not render, but returns a positive mask when drawing is called for.
*/
#ifdef _CRTNOP
_CRTNOP
#endif
int tBoxPinView(textBoxObj*);
/*
 tBoxSetMargin
	Sets the text boundary based on the outer bounds and passed margins.
	These values can be inverted/negative to draw outside the erased bounds.
  Result:
	1 on success; 0 if the new text bounds are empty.
*/
int tBoxSetMargins(textBoxObj *tbox,
	const rect16 *margins, /* can be NULL (no margins) */
	boolean setWrap); /* optional: set maxWidth to txbounds width */
/*
 tVisLine
	Finds the first visible line-run in a list.
  Notes:
	NULL will be returned if 'y' is past the visible range.
*/
const tLineRun *tVisLine(const tLineRun *s,
	int lineCount,
	int y);
int tBoxLineScrollV(const textBoxObj *tbox,
	int amt);
/*
 tBoxCharLine
	Calculates the measured caret line for a character.
  Result:
	Pointer to the line, NULL or a temporary pointer.
	(The result is invalid if lineCount is not positive!)
*/
const tLineRun *tBoxCharLine(textBoxObj*,
	int32 offs,
	ufixed *x, /* unscrolled box coordinate; can be NULL */
	int *top, /* can be NULL */
	int *bottom); /* can be NULL */
/*
 tBoxCharLineIndex
	Uses tBoxCharLine to get the line index for a character.
	It can also be used to scan to the earliest paragraph.
  Result:
	The zero-based index, or -1 on error.
*/
int tBoxCharLineIndex(textBoxObj *tbox,
	int32 offs,
	int *loffs, /* can be NULL */
	int *bottom, /* can be NULL */
	boolean paragraph);
/*
 tBoxPtToChar
	Finds the character for an unscrolled box coordinate.
	This routine rounds to the nearest character edge.
*/
int32 tBoxPtToChar(textBoxObj*,
	ufixed x,
	int y);
/*
 tboxSelRgn
	Generates a region for the selection, clipped vertically.
	Returns NULL if there is no applicable or visible selection beyond a caret.
*/
SLRgn *tBoxSelRgn(textBoxObj*);
/*
 tBoxCharStyle
	Returns the style for a character.
	If there are no styles, the default will be generated.
*/
const tStyleRun *tBoxCharStyle(textBoxObj*,
	int32 i,
	int *remCount); /* remaining run count; can be NULL */

/*
 tBoxCaptureStyles
	Counts and makes a malloc copy of the style runs between ss and se.
	Offsets are adjusted for the resulting range.
*/
int tBoxCaptureStyles(textBoxObj *tbox,
	int32 ss,
	int32 se,
	tStyleRun **runs); /* can be NULL */
/* a variant for handling binary complement */
/* note that ss is used twice */
#define tBoxCopyStyles(tb,ss,se,rs) tBoxCaptureStyles(tb,(ss)^((ss)>>31),se,rs)

int32 tBoxSearch(textBoxObj *tbox,
	int32 start,
	const char *searchStr,
	int32 z,
	uint16 flags);

 #ifdef MTBOX_CLIPBOARD
  #ifdef _CRTNOP
  _CRTNOP
  #endif
int SetClipboard_MacRoman(const char *s,
	int32 z);
  #ifdef _CRTNOP
  _CRTNOP
  #endif
char *GetClipboard_MacRoman(int32 *z,
	boolean stripCRLF);

int ClipMenuSelect(uint16 flags,
	short x,
	short y,
	const char *extension);
 #endif

 #ifdef __MGRAF_INTERNAL
void _tBoxDrawSelect(textBoxObj *tbox,
	int32 oldss,
	int32 oldse);
 #endif
#endif
