/*
 mwind.h
  virtual window/dialog and control include

 * You can redistribute this file and/or modify it under the
 * terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.  You should have
 * received a copy of the license along with this file.

 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See LICENSE.txt more details.
 */

#ifndef __MWIND__
 #define __MWIND__
 #include "mgraf.h"

/* ---- constants */

#define wDialogKind		2
#define wUserKind		8

#define ctlBoundsMaxOutset	3

enum
{ /* window/control colors */
	wCFrame,
	wCTitle,
	wCContent
};

enum
{
	winTypeDocument,
	winTypeModal,
	winTypePlainDialog,
	winTypeShadow,
	winTypeNoGrowDoc,
	winTypeMovableModal
};

enum
{			/* data input	result */
	wcMsgDraw,	/* part (0=all)	visibility */
	wcMsgGetType,	/* none		type index, below */
	wcMsgTest,	/* pt2d16*	label part for loc */
	wcMsgWTest,	/* pt2d16*	label part for wide-ranging loc */
	wcMsgClick,	/* pt2d16*	successful label part */
	wcMsgDblClick,	/* pt2d16*	successful label part */
	wcMsgDragMeas,	/* pt2d16*	new value */
	wcMsgMarkMeas,	/* value	marker/thumb pos. (pt2d16) */

	wcMsgDispose,	/* none */
	wcMsgCalc,	/* ctl-dependent */
	wcMsgInBounds,	/* rect16*	may differ from control bounds */
	wcMsgSetBounds,	/* rect16*	when wSetControlBounds is called */
	wcMsgIdle,	/* none */
	wcMsgHover,	/* none		preferred cursor ID (negative for none) */
	wcMsgWheel,	/* loword:amount; hiword:flags */
	wcMsgReadValue,	/* double*	read primary as a number */

	wcMsgFocus,	/* flags */
	wcMsgDragDrop,
	wcMsgDragEnter,
	wcMsgDragLeave
};

enum
{
	ctlTypePushBtn,
	ctlTypeCheckBox,
	ctlTypeRadioBtn,
	ctlTypeScrollBar,
	ctlTypePopUpMenu,
	ctlTypeTab,
	ctlTypeStaticText,
	ctlTypeEditText,
	ctlTypePicture,
	ctlTypeIcon,
	ctlTypePixMap
};

enum
{
	wcMDirectionV,
	wcMDirectionH,

	wcMFlagDirectionMask		= 0x00FF,
	wcMFlagModifiersMask		= 0xFF00,

};

enum
{
	wcFFlagGain			= 0x1,	/* otherwise lost */
	wcFFlagResetPosition		= 0x2
};

#ifndef __QUICKDRAW__
enum
{
	kControlNoPart,
	kControlLabelPart,
	kControlMenuPart,
	kControlTrianglePart		= 4,
	kControlEditTextPart,
	kControlPicturePart,
	kControlIconPart,
	kControlClockPart,
	kControlButtonPart		= 10,
	kControlCheckBoxPart,
	kControlRadioButtonPart		= kControlCheckBoxPart,
	kControlUpButtonPart		= 20,
	kControlDownButtonPart,
	kControlPageUpButtonPart,
	kControlPageDownButtonPart,
	kControlListBoxPart,
	kControlListBoxDoubleClickPart,
	kControlImageWellPart,
	kControlRadioGroupPart,
	kControlIndicatorPart		= 129,
	kControlDisabledPart		= 254,
	kControlInactivePart
};

enum
{
	kControlCheckBoxCheckedValue	= 1,
	kControlCheckBoxMixedValue
};
#endif

/* ---- typedefs */

#include <pshpack2.h>

#define DEF_WIN_RTN(n)		int n(windowRef win, int msg, const void *data)
#define TYP_WIN_RTN(n)		int(*n)(struct windowRecord*, int, const void*)

typedef struct windowRecord
{
	grafPort	port;
	rect16		bounds, frame, updateRect;
	SLRgn		*clipSave, *updateRgn;
	int8		kind;
	uint8		visible; /* on the screen, takes clicks; 2=translucent */
	boolean		hilite;
	boolean		goAway;	/* has a closebox */
	void		*refCon;
	uint8		title[256]; /* Pascal string */

	struct windowRecord	*above, *below;
	/* ctlList is root */
	struct ctlObj	*ctlList, *ctlFocus, *ctlDefault, *ctlCancel;

	TYP_WIN_RTN(rtn);

	void		*themeData;
	/* palette for indexed mode colors requested for display when activated */
	//cTab64	*palette;
} windowRecord, *windowRef;

typedef TYP_WIN_RTN(winRoutine);

/* controls */

#define DEF_CTL_RTN(n)		int32 n(ctlObj *ctl, int msg, const void *data)
#define TYP_CTL_RTN(n)		int32(*n)(struct ctlObj*, int, const void*)
#define DEF_TRK_RTN(n)		void n(ctlObj *ctl, int part, int32 value)
#define TYP_TRK_RTN(n)		void(*n)(struct ctlObj*, int, int32)

typedef TYP_CTL_RTN(ctlRoutine);

typedef struct ctlObj
{
	struct ctlObj	*next; /* in the window's chain */
	struct ctlObj	*group; /* if not NULL, a lead in a group */
	windowRef	wind; /* owner */
	uint8		title[256]; /* Pascal string */
	rect16		bounds;
	int32		min, max, value;
	void		*refCon;
	uint8		visible, groupCount;
	uint16		hilite; /* [user.8] [labelHilite.8] */
	uint16		autoHiliteTime; /* for focus/idling */
	uint16		txFont, txSize, txStyle;
	int16		labelIndent;

	TYP_CTL_RTN(rtn);
	TYP_TRK_RTN(trk);
	void		*data;
} ctlObj;

typedef struct ctlDragData
{
	void		*originData;
	int		originType, originPart;
	void		*data;
	int32		dataLen;
} ctlDragData;

typedef struct wcItem
{
	rect16		bounds;
	const uint8	*title;
	int8		type, hilite;
	int32		maximum;
} wcItem, wcTemplate;

#ifndef __QUICKDRAW__
 typedef boolean (*wModalFilter)(windowRef, struct eventRecord*, int16 *itemHit);
#endif

#include <poppack.h>

/* ---- globals */

extern windowRecord	screen_desktop_window;

/* ---- prototypes/mwind.c */

int wShowWindow(windowRef wind,
	boolean visible);
int wSizeWindow(windowRef wind,
	int16 width,
	int16 height,
	boolean update);
windowRef wFrontWindow(void);
int wOpenWindow(windowRef newWin,
	windowRef behind,
	int16 procID);
void wCloseWindow(windowRef deadWin);
void wDisposeWindow(windowRef deadWin);
/* always created hidden */
windowRef wNewWindowFromTemplate(const wcTemplate*,
	windowRef behind);

#ifdef _CRTNOP
_CRTNOP
#endif
int wPaintOne(windowRef win,
	const rect16 *inval);
#ifdef _CRTNOP
_CRTNOP
#endif
int wPaintBehind(windowRef win,
	const rect16 *inval);
void wSendBehind(windowRef win,
	windowRef behind);
void wBeginUpdate(windowRef win);
void wEndUpdate(windowRef win);
void wInvalRect(windowRef win,
	const rect16 *dirtyRect);
void wInvalRgn(windowRef win,
	const SLRgn *dirtyRgn);
void wValidRect(windowRef win,
	const rect16 *updatedRect);
void wValidRgn(windowRef win,
	const SLRgn *updatedRgn);
void wCalcVis(windowRef sending);

/* ---- prototypes/mctl.c */

ctlObj *wNewControl(windowRef wind,
	const rect16 *bounds,
	const uint8 *title, /* can now be NULL (empty) */
	boolean visible,
	int32 value,
	int32 min,
	int32 max,
	int type, /* see enum above */
	void *refCon);
#ifdef _CRTNOP
_CRTNOP
#endif
int wDisposeControl(ctlObj*);
#ifdef _CRTNOP
_CRTNOP
#endif
int wKillControls(windowRef wind);
#ifdef _CRTNOP
_CRTNOP
#endif
int wDrawControls(windowRef wind);

#ifdef _CRTNOP
_CRTNOP
#endif
int wDraw1Control(ctlObj *ctl);
#ifdef _CRTNOP
_CRTNOP
#endif
int wIdleControls(windowRef wind);
#ifdef _CRTNOP
_CRTNOP
#endif
void wIdleFocus(windowRef wind);
ctlObj *wControlEvent(windowRef wind,
	const struct eventRecord*);

/* returns label part */
int wFindControl(const windowRef wind,
	pt2d16 loc,
	ctlObj **out);
int wFindControlIndex(const windowRef win,
	pt2d16 loc,
	boolean test);
int32 wCountControls(windowRef wind);
ctlObj *wGetIndexedControl(windowRef win,
	int item);
int wGetControlIndex(const windowRef wind,
	const ctlObj *ctl);

#ifdef _CRTNOP
_CRTNOP
#endif
int wSetControlBounds(ctlObj *ctl,
	const rect16*);
#ifdef _CRTNOP
_CRTNOP
#endif
int wSetControlValue(ctlObj *ctl,
	int32 value);
#ifdef _CRTNOP
_CRTNOP
#endif
int wSetControlMinimum(ctlObj *ctl,
	int32 value);
#ifdef _CRTNOP
_CRTNOP
#endif
int wSetControlMaximum(ctlObj *ctl,
	int32 value);
#ifdef _CRTNOP
_CRTNOP
#endif
int wSetControlTitle(ctlObj *ctl,
	const uint8*);
#ifdef _CRTNOP
_CRTNOP
#endif
int wHiliteControl(ctlObj *ctl,
	uint16 hilite);
#ifdef _CRTNOP
_CRTNOP
#endif
int wSelectControlText(ctlObj *ctl,
	int32 selStart,
	int32 selEnd);
#ifdef _CRTNOP
_CRTNOP
#endif
int wSetControlText(ctlObj *ctl,
	const char *s,
	int32 z);
int32 wGetControlText(ctlObj *ctl,
	char *outStr,
	int32 z);

#ifdef _CRTNOP
_CRTNOP
#endif
int wSetControlVisible(ctlObj *ctl,
	boolean visible);
#define wShowControl(c) wSetControlVisible(c,1)
#define wHideControl(c) wSetControlVisible(c,0)

void wCtlNewFocus(ctlObj *ctl,
	boolean focusHomePosition);
void wCtlSelectNext(ctlObj *ctl,
	boolean reverse);

int wCtlRequestDragDrop(const windowRef wind,
	pt2d16 loc,
	ctlObj **out, /* must be valid */
	ctlDragData *dragInfo);

#define defaultBtnAnimDelay 42

#ifdef _CRTNOP
_CRTNOP
#endif
int wCtlAnimHilite(ctlObj *ctl,
	uint8 speed); /* in 2*milliseconds */

#ifdef __MWIND_PRIVATE
#ifdef _CRTNOP
_CRTNOP
#endif
int _imgContColors(cTab64 *ct,
	const rgb48 *cont,
	const rgb48 *back,
	boolean disabled);
void _defMWinModalDraw(const rect16 *box,
	const rgb48 *color,
	boolean fill);
extern const rgb48 _defMWinColors[3];
#endif

 #if !TARGET_OS_MAC && !defined(__QUICKDRAW__) /* QT 3.0+; you never know */
  typedef ctlObj *ControlRef;
  typedef windowRef WindowRef, DialogRef;
  typedef windowRecord DialogRecord;
  typedef grafPort *GrafPtr, *CGrafPtr;
  typedef void *DialogTHndl;

  void MSetPort(GrafPtr);
  void GetPort(GrafPtr*);

  void wModalDialog(wModalFilter filter,
	int16 *itemHit);

  void wMcsDelay(int32 mcs);

  #define GetWindowPort(w) (&(w)->port)
  #define GetWindowFromPort(p) ((struct windowRecord*)(p))
  #define GetDialogWindow(p) (p)
  #define GetWindowDialog(p) (p)
  #define GetWRefCon(p) ((p)->refCon)
  #define SetWRefCon(p,v) (void)((p)->refCon=v)

  #define FrontWindow wFrontWindow
  #define DisposeWindow wDisposeWindow
  #define MacShowWindow(w) (void)wShowWindow(w,1)
  #define HideWindow(w) (void)wShowWindow(w,0)
  #define SendBehind(w) wSendBehind(w,NULL)
  #define BringToFront(w) wSendBehind(w,(windowRef)-1l)
  #define BeginUpdate wBeginUpdate
  #define EndUpdate wEndUpdate
  #define InvalWindowRect wInvalRect

  #define DisposeControl (void)wDisposeControl
  #define KillControls (void)wKillControls
  #define DrawControls (void)wDrawControls
  #define Draw1Control (void)wDraw1Control
  #define IdleControls (void)wIdleControls
  #define FindControl(l,w,c) wFindControl(w,l,c)
  #define SetControlValue (void)wSetControlValue
  #define SetControlMinimum (void)wSetControlMinimum
  #define SetControlMaximum (void)wSetControlMaximum
  #define SetControlTitle (void)wSetControlTitle
  #define HiliteControl (void)wHiliteControl
  #define GetControlValue(c) (0+(c)->value)
  #define GetControlMinimum(c) (0+(c)->min)
  #define GetControlMaximum(c) (0+(c)->max)
  #define GetControlHilite(c) (0+(c)->hilite)
  #define GetControlTitle(c,s) (void)memcpy(s,(c)->title,256)

  void update_screen_window_rect(windowRef win,
	int left,
	int top,
	int right,
	int bottom);
  /* updates clipRgn->bBox */
  void update_screen_window_clip(windowRef win);

 #endif
#endif
