/*
 shellRc.c
  for loading/accessing resources in an MS Windows shell module

    This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

    This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.

    You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define HAVE_OS
#include "my_cseries.h"
#include "rType.h"
#include "error_codes.h"
#include "interpreter.h"
#include "resex.h"

/* ---- code */

static void *GetModuleResourceType(HMODULE module,
	int32 type,
	int32 *z,
	int *_n)
{
	static int16	single[3] = {1}; /* ID 0, no flags or name */
	HRSRC		rc;
	HGLOBAL		rcData;
	void		*s = NULL;
	int		n;

	if (z)
		*z = 0;
	if ((n=rTypeGetIndex(type))<0)
		return NULL;
	n+= 256;
	if (_n)
		*_n = n;
	if ((rc=FindResource(module,MAKEINTRESOURCE(n),RT_RCDATA))==NULL)
	{
		if ((rc=FindResource(module,MAKEINTRESOURCE(1),MAKEINTRESOURCE(n)))!=NULL)
			{s = single; goto getz;}
	} else if ((rcData=LoadResource(module,rc))!=NULL)
	{
		s = LockResource(rcData);
		if (n<6)
			s = NULL; /* too small for valid list */
	getz:	n = SizeofResource(module,rc);
		if (z)
			*z = n;
	}
	return s;
}

int CountModuleResources(HMODULE module,
	int32 type)
{
	const uint8	*s;

	if (!module || (s=GetModuleResourceType(module,type,NULL,NULL))==NULL)
		return 0;
	return *s|s[1]<<8;
}

static void *GetModuleResourceData(HMODULE module,
	const uint8 *s,
	int tx,
	int ix,
	int32 *z,
	int32 *r)
{
	HGLOBAL		rcData;
	HRSRC		rc = NULL;
	void		*h = NULL;
	int8		*p;

	if ((*z=*s++)==0)
	{
		if ((rc=FindResource(module,MAKEINTRESOURCE((ix<0 ? ~ix:ix)+1),
				MAKEINTRESOURCE(tx)))==NULL)
			{h = hNew(1,0); goto end;}
		*z = SizeofResource(module,rc);
	}
	if (ix<0)
		return NULL;
	if ((h=hNew(*z,1))==NULL)
		goto memerr;
	if ((p=hLock(h))==NULL)
	{
		if (r)
			*r =-1;
		goto mf;
	}
	if (rc)
	{
		if ((rcData=LoadResource(module,rc))==NULL)
		{
		memerr: if (r)
				*r = errOutOfMemory;
		mf:	hFree(h);
			return NULL;
		}
		memcpy(p,LockResource(rcData),*z);
	} else
		memcpy(p,s,*z);
end:	if (!(s[-2]&RES_LOCKED_FLAG))
		hUnlock(h);
	return h;
}

void *GetModuleIndResource(HMODULE module,
	int32 type,
	int ix,
	RsrcInfo *info,
	int *_r)
{
	void		*h = NULL;
	const uint8	*s, *e;
	int32		z;
	int		r = errNoRsrcMapEntry, c, x, n;

	if (!module)
		goto end;
	s = GetModuleResourceType(module,type,&z,&n);
	e = s+z, z = 0;
	if (!s)
		goto end;
	x = ix<0 ? ~ix:ix;
	c = *s|s[1]<<8, s+= 2;
	if (x>=c)
		goto end;
	while(x--)
	{
		if (s>=e)
			goto badmap;
		/* ID.s16, name.pstr, flag.b, shortsize.b */
		s+= s[2]+4, s+= *s+1;
	}
	if (info)
	{
		info->offsetOnDisk = 0;
		info->ID = (int16)(*s|s[1]<<8);
		memcpy(info->name,s+2,1+s[2]);
	}
	s+= s[2]+4;
	if (s>=e-*s)
	{
	badmap: r = errReadBadEntryMap;
		goto end;
	}
	h = GetModuleResourceData(module,s,n,ix,&z,&r);
end:	if (_r)
		*_r = r;
	if (info)
		info->sizeOnDisk = z;
	return h;
}

/* 2023.06.22 */
int IterateModuleResources(HMODULE module,
	int32 type,
	void *userData,
	int func(int ID,
		const uint8 *name,
		void *userData))
{
	const uint8	*s, *e;
	int32		z;
	int		r = 0, c;

	if (!module)
		goto end;
	s = GetModuleResourceType(module,type,&z,NULL);
	e = s+z, z = 0;
	if (!s)
		goto end;
	c = *s|s[1]<<8, s+= 2;
	if (func)
	{
		while(--c>=0)
		{
			if (s>=e)
				return errReadBadEntryMap;
			if ((r=func((int16)(*s|s[1]<<8),s+2,userData))<0)
				goto end;
			/* ID.s16, name.pstr, flag.b, shortsize.b */
			s+= s[2]+4, s+= *s+1;
		}
	} else
		while(--c>=0)
		{
			if (s>=e)
				return errReadBadEntryMap;
			/* ID.s16, name.pstr, flag.b, shortsize.b */
			s+= s[2]+4, s+= *s+1;
		}
end:	return r;
}

static const uint8 *getRCEntry(const uint8 *s,
	const uint8 *e,
	const uint8 *name,
	int ID,
	int *r)
{
	int		i, c;

	if ((c=*s|s[1]<<8)==0)
		return NULL;
	s+= 2;
	if (name)
		for(i=0; i<c; ++i)
		{
			if (s>=e)
				{*r = errReadBadEntryMap; return NULL;}
			if (s[2]==*name && !memcasecmp(3+(const char*)s,
					1+(const char*)name,*name))
			{
				*r = i;
				return s;
			}
			/* ID.s16, name.pstr, flag.b, shortsize.b */
			s+= s[2]+4, s+= *s+1;
		}
	else
		for(i=0; i<c; ++i)
		{
			if (s>=e)
				{*r = errReadBadEntryMap; return NULL;}
			if (ID==(int16)(*s|s[1]<<8))
			{
				*r = i;
				return s;
			}
			/* ID.s16, name.pstr, flag.b, shortsize.b */
			s+= s[2]+4, s+= *s+1;
		}
	return NULL;
}

void *GetModuleResource(HMODULE module,
	int32 type,
	const uint8 *name,
	int ID,
	int *_r)
{
	void		*h = NULL;
	const uint8	*s, *e;
	int32		z;
	int		r = errNoRsrcMapEntry, n, ix;

	if (!module || (s=GetModuleResourceType(module,type,&z,&n))==NULL)
		goto end;
	e = s+z;
	if ((s=getRCEntry(s,e,name,ID,&ix))==NULL)
		{r = ix; goto end;}
	s+= s[2]+4, z = 0;
	if (s>=e-*s)
		r = errReadBadEntryMap;
	else
		h = GetModuleResourceData(module,s,n,ix,&z,&r);
end:	if (_r)
		*_r = r;
	return h;
}

int GetModuleResourceIndex(HMODULE module,
	int32 type,
	const uint8 *name,
	int ID)
{
	const uint8	*s;
	int32		z;
	int		r = errNoRsrcMapEntry, n;

	if (!module || (s=GetModuleResourceType(module,type,&z,&n))==NULL)
		return errNoRsrcMapEntry;
	(void)getRCEntry(s,s+z,name,ID,&r);
	return r;
}
