/*
 evaluate.h
  the evaluation switch for interpreter v2.0
*/
#if EVAL_FLOATING && defined(HAVE_COMPLEX_EVAL)
/* macro for nulifying finite vector components if infinites appear */
 #define fixvector(a) do{ \
	if(fabs(creal(a))==INFINITY) { \
		if(fabs(cimag(a))<INFINITY) \
			a=creal(a); \
	} else if(fabs(cimag(a))==INFINITY && fabs(creal(a))<INFINITY) \
		a=cimag(a)*I; \
}while(0)
#else
 #define fixvector(a) (void)(a)
#endif

#ifndef _OPC
 #define _OPC(m)		case _op_##m&0x3F:
#endif
#ifndef _OPD
 #define _OPD(m)		case(_op_##m&0x3F)|0x20:
#endif
/* unary: op b==>a */
case _op_stv:
	if (!(o&0x80))
		goto end;
	/* set */
	s = unpack_eval(s,o,&a);
	/* b would hold reference for the destination but the
	 current implementation doesn't allow this
	*/
	return 0; /* <xstack */
_OPC(ldv) a = b; break;
_OPC(ldi) /* load indirect */
	if (geti)
		a = geti(b);
	break;
_OPC(srt) a = esqrt(b); break; /* square root */
_OPC(abs) evrset(a,eabs(b)); break;
_OPC(neg) evnset(a,b); break;
_OPC(not) evnset(a,b); evradd(a,-1); break; /* binary compliment */
/* POCC fp bugs workaround: use i and if */
_OPC(bno) i = 0; if (evisnil(b)) i = 1; goto sei; /* boolean NOT */
_OPC(dno) i = 0; if (!evisnil(b)) i = 1; goto sei; /* double- boolean NOT */
#ifdef ALLOW_BIN_HI_LO
_OPC(blo) t = b, a = t&0xFF; break;
_OPC(bhi) t = b, a = (t>>8)&0xFF; break;
#endif
/* dyadic */
_OPC(ban) i = 0; if (!evisnil(a) && !evisnil(b)) i = 1; goto sei; /* bool. AND */
_OPC(bor) i = 0; if (!evisnil(a) || !evisnil(b)) i = 1; goto sei; /* bool. OR */
_OPC(and) _OPD(and) evronly2(a,b); t = evreal(b); evrset(a,evintt(a)&t); break; /* bin. AND */
_OPC(ior) _OPD(ior) evronly2(a,b); t = evreal(b); evrset(a,evintt(a)|t); break; /* bin. OR */
_OPC(eor) _OPD(eor) evronly2(a,b); t = evreal(b); evrset(a,evintt(a)^t); break; /* bin. Ex-OR */
_OPC(asr) evronly2(a,b); t = evreal(b); evrset(a,evintt(a)>>t); break; /* arithmetic shift right */
_OPD(asr) evronly2(a,b); t = evreal(a); evrset(a,evintt(b)>>t); break;
_OPC(shr) /* logical shift right */
#if EVAL_FLOATING
	evronly2(a,b); evrset(a,scalb(evreal(a),-(int)evreal(b)));
#else
	a = (unsigned)a>>(unsigned)b;
#endif
	break;
_OPD(shr)
#if EVAL_FLOATING
	evronly2(a,b); evrset(a,scalb(evreal(b),-(int)evreal(a)));
#else
	a = (unsigned)b>>(unsigned)a;
#endif
	break;
_OPC(shl) /* log. shift left */
#if EVAL_FLOATING
	evronly2(a,b); evrset(a,scalb(evreal(a),(int)evreal(b)));
#else
	a = (unsigned)a<<(unsigned)b;
#endif
	break;
_OPD(shl)
#if EVAL_FLOATING
	evrset(a,scalb(evreal(b),(int)evreal(a)));
#else
	a = (unsigned)b<<(unsigned)a;
#endif
	break;
_OPC(add) evvadd(a,b); fixvector(a); break;
_OPD(add) evvadd(b,a); a = b; fixvector(a); break; /* manual order (because of NaNs) */
_OPC(sub) evvsub(a,b); fixvector(a); break; /* subtract */
_OPD(sub) evvsub(b,a); a = b; fixvector(a); break; /* manual order */
case _op_rml: /* evaluation-only */
_OPD(mul)
 #if EVAL_FLOATING && defined(HAVE_COMPLEX_EVAL)
	a = emult(b,a);
	break;
 #endif
_OPC(mul) /* multiply */
 #if EVAL_FLOATING && defined(HAVE_COMPLEX_EVAL)
	a = emult(a,b);
 #else
	a = (evvunsign)a*(evvunsign)b;
 #endif
	break;
_OPC(pow) a = epow(a,b); break; /* exponentiation */
_OPD(pow) a = epow(b,a); break;
_OPC(div) /* division */
#if EVAL_FLOATING
 #ifdef HAVE_COMPLEX_EVAL
	a = edivi(a,b);
 #else
	a/= b;
 #endif
#else
	a = !b ? ~0u:(unsigned)a/(unsigned)b;
#endif
	break;
_OPD(div)
#if EVAL_FLOATING
 #ifdef HAVE_COMPLEX_EVAL
	a = edivi(b,a);
 #else
	a = b/a;
 #endif
#else
	a = !a ? ~0u:(unsigned)b/(unsigned)a;
#endif
	break;
_OPC(dvs) /* signed div. */
	evrset(a,evisnil(b) ? EVAL_HUGEVAL:evintt(evvdiv(a,b)));
	break;
_OPD(dvs)
	evrset(a,evisnil(a) ? EVAL_HUGEVAL:evintt(evvdiv(b,a)));
	break;
_OPC(mod) /* modulus */
#if EVAL_FLOATING
	evrset(a,emod(a,b));
#else
	a = !b ? 0:(unsigned)a%(unsigned)b;
#endif
	break;
_OPD(mod)
#if EVAL_FLOATING
	evrset(a,emod(b,a));
#else
	a = !a ? 0:(unsigned)b%(unsigned)a;
#endif
	break;
_OPC(mds) evronly2(a,b); t = evintt(b); evrset(a,evisnil(b) ? 0:evintt(a)%t); break; /* signed mod. */
_OPD(mds) evronly2(a,b); t = evintt(a); evrset(a,evisnil(a) ? 0:evintt(b)%t); break;

_OPC(clt) evronly2(a,b); i = 0; if (evreal(a)< evreal(b)) i = 1; goto sei; /* lesser */
_OPC(cge) evronly2(a,b); i = 0; if (evreal(a)>=evreal(b)) i = 1; goto sei; /* greater or equal */
_OPC(cgt) evronly2(a,b); i = 0; if (evreal(a)> evreal(b)) i = 1; goto sei; /* greater */
_OPC(cle) evronly2(a,b); i = 0; if (evreal(a)<=evreal(b)) i = 1; goto sei; /* lesser or equal */

_OPC(ceq) i = 0; if (evveq(a,b)) i = 1; goto sei; /* equal */
_OPC(cne) i = 0; if (evvne(a,b)) i = 1; goto sei; /* not equal */
_OPC(cid) i = 0; if (!memcmp(&a,&b,sizeof(a))) i = 1; goto sei; /* identical */
_OPC(cni) i = 0; if (!!memcmp(&a,&b,sizeof(a))) i = 1; goto sei; /* not identical */

_OPC(cnj)
#ifdef HAVE_COMPLEX_EVAL
	a = conj(b);
#endif
	break;

_OPC(cae) i = 0; if (evveq(evdbl32(a),evdbl32(b))) i = 1; /* approx. equal */
	sei: evrset(a,i);
	break;

#undef fixvector
