/*
 iuctable.h

  for a case-insensitive translation table of parsing characters
  word chars include greek letters
*/

/* IUC-remapped */
#define IC_SP		0x00
#define IC_HT		0x01
#define IC_FF		0x02
#define IC_VT		0x03
#define IC_LF		0x04
#define IC_CR		0x05
#define IC_FS		0x06
#define IC_GS		0x07
#define IC_RS		0x08
#define IC_US		0x09
#define IC_SOH		0x0A
#define IC_EOT		0x0B
#define IC_STX		0x0C
#define IC_ETX		0x0D
#define IC_ENQ		0x1C
#define IC_ACK		0x1D
#define IC_BELL		0x1E
#define IC_BKSP		0x1F
#define IC_NULL		0x20

#define UC_SUPR1	0x00B9
#define UC_SUPR2	0x00B2
#define UC_SUPR3	0x00B3
#define UC_SUPR0	0x2070
#define UC_SUPRI	0x2071
#define UC_SUPR9	0x2079
#define UC_SUPRN	0x207F
#define UC_VF1_4	0xBC
#define UC_VF1_2	0xBD
#define UC_VF3_4	0xBE
#define UC_VF0_3	0x2189
#define UC_VF1_3	0x2153
#define UC_VF1_5	0x2155
#define UC_VF1_8	0x215B
#define SUP0		UC_SUPR0-0x4000
#define SUP1		UC_SUPR1-0x1000
#define SUP2		UC_SUPR2-0x1000
#define SUP3		UC_SUPR3-0x1000
#define SUP4		UC_SUPR4-0x4000
#define SUPI		UC_SUPRI-0x4000
#define SUPN		UC_SUPRN-0x4000
#define F1_4		UC_VF1_4-0x1000
#define F1_2		UC_VF1_2-0x1000
#define F3_4		UC_VF3_4-0x1000
#define F0_3		UC_VF0_3-0x4000
#define F1_3		UC_VF1_3-0x4000
#define F1_5		UC_VF1_5-0x4000
#define F1_8		UC_VF1_8-0x4000

#define IC_NBSP		(int16)0xFFFF

/* UCS2 */
#define UC_FDEL		0x7F
#define UC_RLF		0x8D
#define UC_CENT		0xA2
#define UC_POUND	0xA3
#define UC_CRNCY	0xA4
#define UC_YEN		0xA5
#define UC_BVBAR	0xA6
#define UC_SECTN	0xA7
#define UC_DIAER	0xA8
#define UC_COPYR	0xA9
#define UC_F_ORD	0xAA
#define UC_ALDQU	0xAB
#define UC_NOT		0xAC
#define UC_SHYPH	0xAD
#define UC_REGTM	0xAE
#define UC_MACRN	0xAF
#define UC_DEG		0xB0
#define UC_PL_MN	0xB1
#define UC_ACUTE	0xB4
#define UC_MICRO	0xB5
#define UC_PILCR	0xB6
#define UC_MIDDT	0xB7
#define UC_CEDLA	0xB8
#define UC_M_ORD	0xBA
#define UC_ARDQU	0xBB
#define UC_MUL		0xD7
#define UC_SHRPS	0xDF
#define UC_AGRAV	0xE0
#define UC_ATILD	0xE3
#define UC_ADIAE	0xE4
#define UC_ARING	0xE5
#define UC_AELIG	0xE6
#define UC_CCEDL	0xE7
#define UC_EGRAV	0xE8
#define UC_IGRAV	0xEC
#define UC_ETH		0xF0
#define UC_NTILD	0xF1
#define UC_OGRAV	0xF2
#define UC_OTILD	0xF5
#define UC_ODIAE	0xF6
#define UC_DIV		0xF7
#define UC_OSLSH	0xF8
#define UC_UGRAV	0xF9
#define UC_YACUT	0xFD
#define UC_THORN	0xFE
#define UC_YDIAE	0xFF
#define UC_ABREV	0x0102
#define UC_DTLSI	0x0131
#define UC_OELIG	0x0153
#define UC_SCARN	0x0161
#define UC_ZCARN	0x017E
#define UC_FHOOK	0x0192
#define UC_TURNC	0x0254 /* opt+:,c */
#define UC_SCHWA	0x0259 /* opt-shift+:,a */
#define UC_EZH		0x0292 /* opt+:,z */
#define UC_CIRCM	0x02C6
#define UC_CARON	0x02C7
#define UC_BREVE	0x02D8
#define UC_DOTAB	0x02D9
#define UC_RNGAB	0x02DA
#define UC_OGONK	0x02DB
#define UC_SMTLD	0x02DC
#define UC_DBLAC	0x02DD
#define UC_DITNS	0x0385
#define UC_ALPHA	0x03B1
#define UC_PI		0x03C0
#define UC_OMEGA	0x03C9

#define ACUT		UC_ACUTE-0x1000
#define BREV		UC_BREVE-0x1000
#define CARN		UC_CARON-0x1000
#define CDLL		UC_CEDLA-0x1000
#define CRCM		UC_CIRCM-0x1000
#define DBAC		UC_DBLAC-0x1000
#define DIAE		UC_DIAER-0x1000
#define DTAB		UC_DOTAB-0x1000
#define MCRN		UC_MACRN-0x1000
#define OGNK		UC_OGONK-0x1000
#define RNAB		UC_RNGAB-0x1000
#define SMTL		UC_SMTLD-0x1000

#define ARNG		UC_ARING-0x1000
#define ATLD		UC_ATILD-0x1000
#define OTLD		UC_OTILD-0x1000
#define NTLD		UC_NTILD-0x1000
#define OSLS		UC_OSLSH-0x1000
#define CCED		UC_CCEDL-0x1000
#define SCAR		UC_SCARN-0x1000
#define ZCAR		UC_ZCARN-0x1000
#define SHPS		UC_SHRPS-0x1000
#define THRN		UC_THORN-0x1000
#define DTLI		UC_DTLSI-0x1000
#define ETH_		UC_ETH-0x1000
#define FHOO		UC_FHOOK-0x1000
#define FORD		UC_F_ORD-0x1000
#define MORD		UC_M_ORD-0x1000

#define YDIA		UC_YDIAE-0x1000
#define YACU		UC_YACUT-0x1000
#define AELG		UC_AELIG-0x1000
#define OELG		UC_OELIG-0x1000

#define OMGA		UC_OMEGA-0x1000

#define UC_NDASH	0x2013
#define UC_MDASH	0x2014
#define UC_DBLLL	0x2017
#define UC_LO9SQ	0x201A
#define UC_LO9DQ	0x201E
#define UC_HELIP	0x2026
#define UC_BULLT	0x2022
#define UC_DAGGR	0x2020
#define UC_EURO		0x20AC
#define UC_PSETA	0x20A7
#define UC_LDQUO	0x201C
#define UC_RDQUO	0x201D
#define UC_LSQUO	0x2018
#define UC_RSQUO	0x2019
#define UC_PERML	0x2030
#define UC_PERTT	0x2031
#define UC_ALSQU	0x2039
#define UC_ARSQU	0x203A
#define UC_FRACS	0x2044
#define UC_TM		0x2122
#define UC_OHM		0x2126
#define UC_PARD		0x2202
#define UC_INCR		0x2206
#define UC_PROD		0x220F
#define UC_SUM		0x2211
#define UC_RAD		0x221A
#define UC_INF		0x221E
#define UC_INTSC	0x2229
#define UC_INTGR	0x222B
#define UC_ALMEQ	0x2248
#define UC_INEQ		0x2260
#define UC_TRIPB	0x2261
#define UC_LSOEQ	0x2264
#define UC_GTOEQ	0x2265
#define UC_RNOT		0x2310
#define UC_LOZNG	0x25CA
#define UC_LOGOP	(int16)0xF8FF
#define UC_fiLIG	(int16)0xFB01
#define UC_flLIG	(int16)0xFB02
#define UCFDIG_0	(int16)0xFF10 /* full-width variants */
#define UCFDIG_9	(int16)0xFF19
#define UCFLAT_AT	(int16)0xFF20
#define UCFLAT_A	(int16)0xFF21 /* these need to be capital to be < `_` */
#define UCFLAT_F	(int16)0xFF26
#define UCFLAT_Z	(int16)0xFF3A
#define UCFUNDSC	(int16)0xFF3F /* _ */
#define UCFGRAVE	(int16)0xFF40
#define UCFTILDE	(int16)0xFF5E
#define UC_HANGUL_JS_I	(int16)0xFFDC
#define UC_INVAL	(int16)0xFFFD

#define IC_ISWCH(x)	(_interpUCType[(uint8)(x)]<UC_INVAL) /* word char */
#define IC_ISWID(x)	(_interpUCType[(uint8)(x)]<=UCFUNDSC) /* v1 ID char */
#define IC_ISALN(x)	(_interpUCType[(uint8)(x)]<=UCFLAT_Z) /* alphanumeric */
#define IC_ISHEX(x)	(_interpUCType[(uint8)(x)]<=UCFLAT_F)
#define IC_ISDIG(x)	(_interpUCType[(uint8)(x)]<=UCFDIG_9)
#define IC_ISLAT(x)	(_interpUCType[(uint8)(x)]+0u-UCFLAT_A<26) /* Latin/no accent */
#define IC_ISGRK(x)	(_interpUCType[(uint8)(x)]+0u-UCFALPHA<25) /* Greek/no accent */
#define IC_ISEOL(x)	(_interpUCType[(uint8)(x)]+0u-IC_FF<IC_US-IC_FF) /* py */
#define IC_ISWSP(x)	(_interpUCType[(uint8)(x)]+0u<=IC_NULL)
#define IC_ISBLK(x)	(_interpUCType[(uint8)(x)]+0u<IC_FS) /* blank */
extern const int16 _interpUCType[256];
