#if 0
C -o script script.o var.c.o interpreter.c.o -s Interpreter=Main -di interpreter.h -i :: -d mac -d EVAL_FLOATING -d ALLOW_RUNON_WSP -d ALLOW_ELITERALS -d ALLOW_ESCNL -d ALLOW_REP {"Parameters"}
Exit 0
#endif
/*
 script.c
  basic interpreter (no functions yet)

  Created: 2011.08.20
  Version: 1.9.9f2
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifndef PARSE_ONLY_REAL
 #include <math.h>
#endif
#if EVAL_FLOATING && defined(HAVE_COMPLEX_EVAL)
 #include <complex.h>
#endif

#define INTERP_BASIC_SCRIPT_VARS

#include "interpreter.h"
#include "iuctable.h"
#include "ftrig.h"

/* ---- general */

void global_idle_proc(void);

/* ---- script */

const char *_interp_err_strs[30] =
{
	"syntax error",
	"memory error",
	"read error",
	"write error",
	"incomplete expression",
	"interrogator already open",
	"bad/missing identifier",
	"handler not supplied",
	"stack over/underflow",
	"value out of range",
	"comment still open",
	"unterminated string",
	"identifier expected here",
	"internal op error",
	"cannot place labels here",
	"language not supported",
	"no such macro",
	"cannot optimize this",
	"value already set to something else",
	"destination is immutable",
	"data too large for store",
	"no such struct/list field",
	"no expression for condition",
	"improper closing symbol",
	"unsupported value",
	"unknown symbol",
	"missing keyword",
	"superfluous keyword",
	"expected boolean",
	"block still open"
};

evalvalue __cdecl eval_rindir(evalvalue v)
{
	v = ((int)v&0xFFFF)*12345;
	return v;
}

const char __cdecl *eval_getv(const char *s,
	const char *e,
	evalvalue *i,
	uint8 **u)
{
	const uint8	*d;
	int32		x;
	uint8		h;

	if (!u)
		return NULL;
	d = *(uint8**)u;
	e = parse_wordbreak(s,e,&h,UCFDIG_0,UCFUNDSC), ++s;
	if ((x=e-s)>255u)
		return NULL; /* bad length */
	if ((x=get_hbuf_entry(s,x,h,d,sizeof(evalvalue)+1))<=0)
		return NULL;
	if (i)
		memcpy(i,d+x+1,sizeof(evalvalue));
	return e;
}

const char __cdecl *eval_setv(const char *s,
	const char *e,
	evalvalue i,
	int *err,
	uint8 **u)
{
	uint8		*d, h;
	int32		x, z;

	if (!u)
	{
		if (err)
			*err = _e_write_error;
		return NULL;
	}
	d = *u;
#ifndef PARSE_NO_CONSTANTS
	if (e==(char*)-1)
	{
		e = parse_wordbreak(s,e,&h,UCFDIG_0,UCFUNDSC), ++s;
		if (*e=='~')
			++e;
	} else
#endif
	e = parse_wordbreak(s,e,&h,UCFDIG_0,UCFUNDSC), ++s;
	if ((z=e-s)>255u)
		return NULL; /* bad length */
	if ((x=get_hbuf_entry(s,z,h,d,sizeof(evalvalue)+1))>=0)
	{
		if (!x)
			return NULL;
		d+= x;
#ifndef PARSE_NO_CONSTANTS
		if (d[-1]=='~')
		{
			*err = _e_lvalue;
			return NULL;
		}
#endif
	} else
	{ /* (-x) abs. end offset; automagic +1 includes null term. */
		if ((d=realloc(d,2+1+sizeof(evalvalue)+z-x))==NULL)
		{
			if (err)
				*err = _e_memory_error;
			return NULL;
		}
		*u = d, d-= x;
		d[-1] = h, *d++ = (uint8)z;
		memcpy(d,s,z), d+= z, d[sizeof(evalvalue)+1] = 0;
	}
	memcpy(d+1,&i,sizeof(evalvalue));
	return e;
}

void interp_init_basic_vars(uint8 **vbuf,
	int *err)
{
	eval_setv("C~",(char*)-1,299792458,err,vbuf); /* light in vaccuum, m/sec */
	eval_setv("pi~",(char*)-1,M_PI,err,vbuf); /* circum./diam. */
	eval_setv("Pi~",(char*)-1,M_PI,err,vbuf); /* circum./diam. */
	eval_setv("E~",(char*)-1,M_E,err,vbuf);
	eval_setv("phi~",(char*)-1,M_PHI,err,vbuf);
	eval_setv("Phi~",(char*)-1,M_PHI,err,vbuf);
	eval_setv("inf~",(char*)-1,INFINITY,err,vbuf);
	eval_setv("INF~",(char*)-1,INFINITY,err,vbuf);
	eval_setv("Inf~",(char*)-1,INFINITY,err,vbuf);
	eval_setv("INFINITY~",(char*)-1,INFINITY,err,vbuf);
	eval_setv("nan~",(char*)-1,NAN,err,vbuf);
	eval_setv("NAN~",(char*)-1,NAN,err,vbuf);
	eval_setv("NaN~",(char*)-1,NAN,err,vbuf);
	eval_setv("true",(char*)-1,1,err,vbuf);
	eval_setv("TRUE",(char*)-1,1,err,vbuf);
	eval_setv("True",(char*)-1,1,err,vbuf);
	eval_setv("false",(char*)-1,0,err,vbuf);
	eval_setv("FALSE",(char*)-1,0,err,vbuf);
	eval_setv("False",(char*)-1,0,err,vbuf);
	eval_setv("_VERSION~",(char*)-1,(INTERP_VERS>>8)
		+(INTERP_VERS>>4&15)*.1+(INTERP_VERS&15)*.01,err,vbuf);
#ifdef HAVE_COMPLEX_EVAL
	eval_setv("I~",(char*)-1,I,err,vbuf);
	eval_setv("infI~",(char*)-1,INFINITY*I,err,vbuf);
	eval_setv("InfI~",(char*)-1,INFINITY*I,err,vbuf);
#endif
}

#if defined(INTERP_STANDALONE) || defined(INTERP_BASIC_SCRIPT_FILE)
//s = malloc(0);
//while(z=~0,l=x,f && (o=ftell(f),s=freadline(f,&z,s,NULL,&x,'\\'))!=NULL)

//#include "eval_op.h"

#if INTERP_VERS>0x199
//((('a'-96<<10)|('d'-96<<5)|('d'-96))<<8)|_op_add,
//((((uint64)'s'-96<<35)|((uint64)'u'-96<<30)|('b'-96<<25)|('t'-96<<20)|('r'-96<<15)|('a'-96<<10)|('c'-96<<5)|('t'-96))<<8)|_op_sub,

const uint64	eval_moskey[] =
{
	((('i'-96<<5)|('f'-96))<<8)|_lexk_if_t,
	((('t'-96<<15)|('h'-96<<10)|('e'-96<<5)|('n'-96))<<8)|_lexk_then,
	0
};
#endif

long evaluate_script(char *s,
	const char *e,
	long li,
	evalstate *state,
	void (__cdecl reportError)(int err,
		char *s,
		long z,
		int32 r,
		int li,
		void *userData),
	int (__cdecl reportTrace)(char *s,
		long z,
		int li,
		const evalstate *state,
		void *userData),
	void *userData)
{
	char		*le, *lasts;
	long		lastl, r;
	uint32		x;
	int		err = 0;

	for(x=li,lasts=NULL,lastl=-1; s<e; global_idle_proc())
	{
		uint32		z = sreadline(s,e-s,&x,'\\');
//printf("%d\n",l);
		if (reportTrace && (!(state->flags&_within_longcomment)
				|| ((le=memchr(s,'#',z))!=NULL && le>s && le[-1]==']'))
				&& z && reportTrace(s,z,li,state,userData)<0)
			return ~li;
		le = s+z, r = evaluate_line(s,le,&err,state);
		if (err)
			goto erro;
		if (r>=_e_repeat_end)
		{
			if (r==_e_return)
				break;
			if (r==_e_repeat)
				{x = li; continue;}
			if (r==_e_repeat_last)
			{
				if (!lasts)
					goto expo;
				s = lasts, x = lastl;
				continue;
			}
			if (!lasts)
			{
			expo:	r = 0, err = _e_no_expr;
			erro:	reportError(err,s,z,r,li,userData);
				goto end;
			}
		} else
		{
			if (s+r<e && s[r]==',')
				{reportError(_e_general_error,s,z,r,li,userData); goto end;}
			if (r) /* substantive line */
				lasts = s, lastl = li;
		}
		li = x, s = le+1;
	}
end:	return li;
}
#endif

/* ---- standalone executable */

#ifdef INTERP_STANDALONE
char		mLocaleNPtChar = '.', mLocaleNSepChar = '_';

size_t swr_fputc(const void *p,
	size_t z,
	void *dst)
{
(void)z;
	return fputc((int)p,(FILE*)dst)>=0;
}

/* converts tabs to spaces so the pointer (^) places right */
static void untab(char *s,
	const char *e)
{
	if (!s)
		return;
	while(s<e)
		if (*s++=='\t')
			s[-1] = ' ';
}

static void __cdecl report_error(int err,
	char *s,
	long z,
	int32 r,
	int li,
	void *userData)
{
(void)userData;
	untab(s,s+z);
	if (err==_e_param_separation)
		err = _e_general_error;
	if (err<0 && (err=~err)<GET_ARRAY_COUNT(_interp_err_strs))
		(void)fprintf(stderr,
"### %.*s\n"
"### % *^\n"
"### %s @%d:%d\n",
			z,s,r+1,_interp_err_strs[err],li,r);
}

static int delim(const char *s,
	const char *e)
{
	while(s<e)
	{
		uint8		c = *s++;

		if ((c-='\t')!=0 && c!=' '-'\t')
			{--s; break;}
	}
	return s+3>e || *s!='>' || s[1]!=';';
}

static void dumpIDs(const uint8 *l,
	int p)
{
	char		c;

	if (!l)
		return;
	(void)fputc('\n',stdout);
	for(; (c=*l++)!=0; l+=sizeof(evalvalue))
	{
		char		b[64];
		const char	*s;
		evalvalue	v;
		int		z = *l++;

		s = (const char*)l, l+= z, ++l;
		if (l[-2]=='~')
			continue;
		memcpy(&v,l,sizeof(v));
		c&= 0x3F, c|= 0x40;
		sprint_evalue(b,v,p);
		(void)fprintf(stdout,"%c%.*s%* = %s\n",
			c,z,s,9-MIN(z,9),b);
	}
}

static long interp_file_script(FILE *f,
	char *s,
	int8 dump)
{
	evalstate	state;
	uint8		*vbuf = NULL;
	int32		z = 0;
	int		r = 0;

	interp_init_basic_vars(&vbuf,&r);
#if TEST_PARSER
	r = parse_number("inf",(char*)-1,vbuf,1);
	printf("%g %d\n",
		*(double*)vbuf,r);
	exit(0);
#endif
	if (r)
		return r;
	if (f)
		s = freadfile(f,&z,f==stdin ? delim:NULL);
	else
		z = strlen(s);
	memset(&state,0,sizeof(state));
	state.flags = _allow_qrepeat|_CPEMDAS_ordering|_allow_approxeq;
	state.getv = eval_getv;
	state.setv = eval_setv;
	state.geti = eval_rindir;
	state.vbuf = &vbuf;
	if (!s)
		r = z ? _e_memory_error:0;
	else
	{
		r = evaluate_script(s,s+z,1,&state,report_error,NULL,NULL);
		if (state.flags&_within_longcomment)
			(void)fprintf(stderr,
	"### %s\n",
				_interp_err_strs[~_e_open_comment]);
		else
		{
			char		b[64];

			(void)sprint_evalue(b,state.result,16);
			printf(
	"# result: %s\n"
	"# %d %s\n",
				b,r,r==1 ? "line":"lines");
			r = 0;
		}
	}
	if (dump)
		dumpIDs(vbuf,dump);
	//eval_free(vbuf);
	free(vbuf);
	if (f)
		free(s);
	return r;
}

#if TARGET_OS_MAC
 #define commandPeriod()		(*((volatile char*)0x179)&*((volatile char*)0x17A))
#else
 #define commandPeriod()		0 /* you lose... */
#endif

static uint8	user_abort, modal;

void global_idle_proc(void)
{
	if (!commandPeriod() && !user_abort)
		return;
	if (user_abort && !modal)
	{
		(void)fputs("\n# User abort.\n",stderr);
		exit(1);
	}
	user_abort = 1;
}

int firstdig(const char *s)
{
	while(*s && (*s=='0' || *s=='.'))
		++s;
	return *s;
}

int main(int argc,
	char **argv)
{
	FILE		*f = NULL;
	const char	*fn = NULL;
	char		**a, *s, *str = NULL;
	int		r = 0;
	uint8		dump = 0, infile = 0;

	if (argc<=0 || (a=argv)==NULL)
		return 0;
	if (argc==1)
	{
		(void)fprintf(stderr,"### usage: %s filename\n",
			*argv);
		(void)fputs(
"#\n"
"#  -            use stdin\n"
"#  -D           dump IDs\n"
"#  -E<expr>     evaluate expression\n"
"#\n",
			stderr);
		return 1;
	}
	while(--argc && (s=*++a)!=NULL)
		if (*s=='-')
		{
			if (!s[1])
			{
				if (f || str)
					goto opn;
				f = stdin, s = "dev::stdin";
				infile = 1;
			} else if (s[1]=='E')
			{
				if (f || str)
					goto opn;
				str = s+2;
			} else if (s[1]=='D')
				dump =-(s[2]=='P');
			else
			{
				(void)fprintf(stderr,"### %s - unknown option '%s'.\n",
					*argv,s);
				return 1;
			}
		} else if (infile)
		opn:	(void)fprintf(stderr,"### %s - one input at a time.\n",
				*argv);
		else if ((f=fopen(fn=s,"r"))!=NULL)
			infile = 1;
		else
		{
			(void)fprintf(stderr,"### %s - could not get/open '%s'.\n",
				*argv,s);
			return 1;
		}
	if (!str && !fn)
	{
		(void)fprintf(stderr,"### %s - no input.\n",
			*argv);
		return 1;
	}
#if !TARGET_API_MAC_CARBON && TARGET_OS_MAC
	if (((volatile char*)0x174)[60>>3]&0x80>>(60&7)) /* keymap - control */
		Debugger();
#endif
	r = interp_file_script(f,str,dump);
	if (f!=stdin)
		(void)fclose(f);
	if (r)
	{
		if (r<0)
			(void)fprintf(stderr,"### %s - fatal error (%d).\n",
				*argv,r);
		else if (infile)
			(void)fprintf(stderr,
"File \"%s\"; line %d\n",
				fn,r);
		r = 1;
	}
	return r;
}
#endif
