/*
 test.c
*/

#ifdef TEST_TIME
 #if TARGET_OS_MAC
  #define TIME_CNT 10000
  #define TIME_MSC(v) (v*1000)
  #define TIME_MSG "%d mcs computing 10,000 usqrt"
  #if TARGET_OS_MAC && TARGET_CPU_68K && !TARGET_RT_MAC_CFM
   #pragma parameter __D0 getTime()
   extern uint32 getTime(void) = {0xA193};
  #else
   #include <Timer.h>
   static uint32 getTime(void)
   {
	uint64		v;

	Microseconds((UnsignedWide*)&v);
	return int64lo(v);
   }
  #endif
  #define Sleep(n) do{int32 _lt=getTime()+TIME_MSC(n); \
	while((int32)getTime()-_lt<0) ; \
  }while(0)
 #else
  #include <windows.h>
  #define TIME_CNT 10000000
  #define TIME_MSC
  #define TIME_MSG "%d ms computing 10,000,000 usqrt"
  #define getTime GetTickCount
 #endif
 #include <stdio.h>
/* per million, on a 2.593GHz Pentium 4
  usqrt:	C ms / clocks	Asm ms / cyc	comment
	FPU	-		78-88 / ~227	larger nos. take longer
	IU	54.8 / 142	32.7 / 85	- Asm less than half of usqrt/FPU!
  usqrt64:	C
	FPU	-		100 / 259
	IU	118.7 / 308	64 / 166	- puts the FPU method to shame
*/
int main(int argc,
	char **argv)
{
#pragma unused(argv)
	uint64		v;
	int32		x = 0, ltick;

	if (argc<1)
		return argc;
 #if TARGET_OS_MAC
	remove("stderr");
	if (((volatile char*)0x174)[60>>3]&(128>>(60&7))) /* keymap - control */
		Debugger();
 #endif
	/* accuracy tests */
	int64sethl(v,-1u,-1u);
	if (usqrt64(v)!=-1u)
		x = 1;
	int64sethl(v,-2u,0);
	if (usqrt64(v)!=-1u)
		x|= 2;
	int64sethl(v,-3u,3);
	if (usqrt64(v)!=-1u)
		x|= 4;
	int64sethl(v,-3u,2);
	if (usqrt64(v)!=-2u)
		x|= 8;
	int64sethl(v,0xFFFEFFFFu,-1u);
	if (usqrt64(v)!=0xFFFF8000u)
		x|= 16;
	int64sethl(v,0x33333333,0x33333333);
	if (usqrt64(v)!=0x727C9717)
		x|= 32;
	if (x)
		fprintf(stderr,"## usqrt64 is not producing accurate results! (x%X)\n",
			x);
	printf("waiting 200 ms for any paging to complete...\n\n");
	Sleep(1);
	for(ltick=getTime(); getTime()-ltick<TIME_MSC(200); )
		;
	for(x=0,ltick=getTime(); x<TIME_CNT; ++x)
	{
		int64sethl(v,x,0);
		usqrt64(v);
	}
	ltick-= getTime();
	printf(
#ifdef NO_FPU
TIME_MSG "/IU\n",
#else
TIME_MSG "/FPU\n",
#endif
		-ltick);
	return 0;
}
#elif defined(MAKE_CSRHLUT)
#include <math.h>
#include <stdio.h>
#include <fenv.h>
int main(int argc,
	char **argv)
{
	int		x;

	fputs(
"/* short cosine (64 entries) and square root half table */\n"
"const uint16	csrhlut[256] =\n"
"{\n"
"	",
		stdout);

	for(x=0; x<64; ++x)
		printf(x&7 ? "%u,":"\n\t%u,",
			(int)(cos(x*((1+.57079632679489662)/64))*32768+.5));
	for(x=64; x<255; ++x)
		printf(x&7 ? "%d,":"\n\t%d,",
			x<64 ? 0:(uint32)(sqrt((x+.5)/256.)*16384+1.5)*4);
	fputs("65535\n};\n",stdout);
	return 0;
}
#elif defined(TEST_FRACT)
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #define M_PI_2		1.57079632679489661923

static void testFractSin(ufract qth)
{
	ifract		qrs;

	qrs = fractsin(qth);
	printf(
"fractsin(0x%X)=%X (%.10g)\n",
		qth,qrs,qrs/1073741824.);
}

int main(int argc,
	char **argv)
{
	
	double		theta = 0;
	ufract		qth;

	if (argc<=0)
		return 0;
	if (--argc)
	{
		theta = atof(argv[1]);
		printf("sin(%.15g)=%.15g\n",
			theta,sin(theta));
		testFractSin(round(theta*(683565275+.57643159)));
		if (argc>1)
		{
			ufixed		numer = atof(argv[2]);
			ufixed		denom = atof(argv[3]);

			qth = ufixmuldiv(theta,numer,denom);
			printf("ufixmuldiv(0x%X,0x%X,0x%X)=0x%X (%.10g as ufixed)\n",
				(ufixed)theta,numer,denom,qth,round(qth*(100000/65536.))/100000);
		}
	} else
	{
		for(qth=0x3FFFFF80; qth<0x40000010; ++qth)
			testFractSin(qth);
		testFractSin(-0x40000000);
		testFractSin(-0x10000000);
		testFractSin(0);
		testFractSin(0x10000000);
		testFractSin(0x15555555);
		testFractSin(0x20000000);
		testFractSin(0x30000000);
		testFractSin(0x517CC1B7);
	}
}
#elif defined(TEST_CONVOLVE)
static ifract impulse[9*3] =
{
 0,
 0,
 0,
 0,
 0,
 0,
 0,
 0,
 0,
-.03125*(1<<30),
 0,
 0.0625*(1<<30),
 .25*(1<<30),
 .4375*(1<<30),
 .25*(1<<30),
 0.0625*(1<<30),
 0,
-.03125*(1<<30),
 0,
 0,
 0,
 0,
 0,
 0,
 0,
 0,
 0,
};

int main(int argc,
	char **argv)
{
	int		x = 0;

	do
	{
		printf("%.10g\n",
			fractZFTransform(impulse+9,impulse+x,
			9)/1073741824.);
	} while(++x<9*3-9);
}
#endif
